@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2007, 2010, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2007, 2011, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -36,12 +36,13 @@
 #include <boost/lambda/bind.hpp>
 
 #include "base/log.h"
-ENABLE_LOG("core")
 
 using namespace bec;
 using namespace wb;
 using namespace base;
 
+DEFAULT_LOG_DOMAIN(DOMAIN_COMMAND_HANDLING)
+
 struct wb::ParsedCommand
 {
   std::string type;
@@ -487,7 +488,7 @@ void CommandUI::add_plugins_menu_items(mforms::MenuItem *parent, const std::stri
     item->set_name(std::string("plugin:").append(*(*iter)->name()));
     item->set_validator(boost::bind(&CommandUI::validate_plugin_command, this, *(*iter)->name()));
     item->validate();
-    scoped_connect(item->signal_clicked(),boost::bind((void(CommandUI::*)(const std::string&))&CommandUI::activate_command, this, item->name()));
+    scoped_connect(item->signal_clicked(),boost::bind((void(CommandUI::*)(const std::string&))&CommandUI::activate_command, this, item->get_name()));
     parent->add_item(item);
   }
 }
@@ -638,6 +639,7 @@ void CommandUI::add_menu_items_for_context(const std::string &context, mforms::M
           if (!plugin.is_valid())
           {
             enabled= false;
+            log_warning("Plugin item %s was not found\n", cmd.name.c_str());
 #ifdef ENABLE_DEBUG
             caption.append(" (invalid)");
 #endif
@@ -699,10 +701,22 @@ static void connect_validate_to_signal(boost::signals2::signal<void ()> &validat
 }
 
 
+void CommandUI::menu_will_show(mforms::MenuItem *item)
+{
+  if (item->get_name() == "open_recent")
+  {
+    item->remove_all();
+    add_recent_menu(item);
+  }
+}
+
+
 mforms::MenuBar *CommandUI::create_menubar_for_context(const std::string &context)
 {
   mforms::MenuBar *menubar = new mforms::MenuBar();
  
+  menubar->signal_will_show()->connect(boost::bind(&CommandUI::menu_will_show, this, _1));
+  
   grt::ListRef<app_MenuItem> main_menu(grt::ListRef<app_MenuItem>::cast_from(_wb->get_grt()->unserialize(make_path(_wb->get_datadir(), "data/main_menu.xml"))));
 
   for (size_t c= main_menu.count(), i= 0; i < c; i++)
@@ -754,7 +768,6 @@ mforms::ToolBar *CommandUI::create_toolbar(const std::string &toolbar_file)
   return create_toolbar(toolbar_file, boost::bind(&CommandUI::activate_command, this, _1));
 }
 
-
 mforms::ToolBar *CommandUI::create_toolbar(const std::string &toolbar_file, const boost::function<void (std::string)> &activate_slot)
 {
   app_ToolbarRef toolbar(app_ToolbarRef::cast_from(_wb->get_grt()->unserialize(_wb->get_grt_manager()->get_data_file_path(toolbar_file))));
@@ -789,63 +802,40 @@ mforms::ToolBar *CommandUI::create_toolbar(const std::string &toolbar_file, cons
     }
     
     mforms::ToolBarItem *tbitem = 0;
-    
+
     if (titem->itemType() == "action")
     {
       tbitem = mforms::manage(new mforms::ToolBarItem(mforms::ActionItem));
       tbitem->set_name(titem->name());
       std::string s = IconManager::get_instance()->get_icon_path(titem->icon());
-#ifdef __APPLE__
-      {
-        std::string ss;
-        ss = titem->icon();
-        if (ss.length() > 4)
-          ss = ss.substr(0, ss.length()-4);
-        ss = IconManager::get_instance()->get_icon_path(ss +"_mac.png");
-        if (!ss.empty())
-          s = ss;
-      }
-#endif
       tbitem->set_icon(s);
       
-      tbitem->scoped_connect(tbitem->signal_activated(),boost::bind(activate_slot, titem->command()));
+      scoped_connect(tbitem->signal_activated(),boost::bind(activate_slot, titem->command()));
     }
     else if (titem->itemType() == "toggle")
     {
       tbitem = mforms::manage(new mforms::ToolBarItem(mforms::ToggleItem));
       tbitem->set_name(titem->name());
       std::string s = IconManager::get_instance()->get_icon_path(titem->icon());
-#ifdef __APPLE__
-      {
-        std::string ss;
-        ss = titem->icon();
-        if (ss.length() > 4)
-          ss = ss.substr(0, ss.length()-4);
-        ss = IconManager::get_instance()->get_icon_path(ss +"_mac.png");
-        if (!ss.empty())
-          s = ss;
-      }
-#endif
       tbitem->set_icon(s);
       
       if (!(titem->altIcon()).empty())
-      {
         s = IconManager::get_instance()->get_icon_path(titem->altIcon());
-#ifdef __APPLE__
-        {
-          std::string ss;
-          ss = titem->altIcon();
-          if (ss.length() > 4)
-            ss = ss.substr(0, ss.length()-4);
-          ss = IconManager::get_instance()->get_icon_path(ss +"_mac.png");
-          if (!ss.empty())
-            s = ss;
-        }
-#endif
-      }
       tbitem->set_alt_icon(s);
       
-      tbitem->scoped_connect(tbitem->signal_activated(),boost::bind(activate_slot, titem->command()));
+      scoped_connect(tbitem->signal_activated(),boost::bind(activate_slot, titem->command()));
+    }
+    else if (titem->itemType() == "segmentedToggle")
+    {
+      tbitem = mforms::manage(new mforms::ToolBarItem(mforms::SegmentedToggleItem));
+      tbitem->set_name(titem->name());
+      tbitem->set_icon(IconManager::get_instance()->get_icon_path(titem->icon()));
+      if (strlen(titem->altIcon().c_str()) > 0)
+        tbitem->set_alt_icon(IconManager::get_instance()->get_icon_path(titem->altIcon()));
+      tbitem->set_checked(titem->initialState() != 0);
+      tbitem->signal_activated()->connect(boost::bind(activate_slot, titem->command()));
+      tbar->add_item(tbitem);
+      continue;
     }
     else if (titem->itemType() == "imagebox")
     {
@@ -858,7 +848,7 @@ mforms::ToolBar *CommandUI::create_toolbar(const std::string &toolbar_file, cons
     {
       tbitem = mforms::manage(new mforms::ToolBarItem(mforms::SearchFieldItem));
 
-      tbitem->scoped_connect(tbitem->signal_activated(),boost::bind(activate_slot, titem->command()));
+      scoped_connect(tbitem->signal_activated(),boost::bind(activate_slot, titem->command()));
     }    
     else if (titem->itemType() == "expander")
     {
@@ -945,11 +935,11 @@ static bool has_active_view(WBContext *wb)
   return (dynamic_cast<ModelDiagramForm*>(wb->get_active_form()) != 0);
 }
 
+//--------------------------------------------------------------------------------------------------
 
 void CommandUI::add_frontend_commands(const std::list<std::string> &commands)
 {
-  for (std::list<std::string>::const_iterator iter= commands.begin();
-    iter != commands.end(); ++iter)
+  for (std::list<std::string>::const_iterator iter= commands.begin(); iter != commands.end(); ++iter)
   {
     // hack
     if (iter->compare("diagram_size")==0
@@ -962,6 +952,18 @@ void CommandUI::add_frontend_commands(const std::list<std::string> &commands)
   }
 }
 
+//--------------------------------------------------------------------------------------------------
+
+/**
+ * Convenience method to remove registered front end commands.
+ */
+void wb::CommandUI::remove_frontend_commands(const std::list<std::string> &commands)
+{
+  for (std::list<std::string>::const_iterator iter= commands.begin(); iter != commands.end(); ++iter)
+    remove_builtin_command(*iter);
+}
+
+//--------------------------------------------------------------------------------------------------
 
 void CommandUI::remove_builtin_command(const std::string &name)
 {
@@ -1057,6 +1059,7 @@ void CommandUI::activate_command(const std::string &command)
   {
 //    bec::UIForm *active_form= get_active_form();
 
+    // XXX: allow parameters also for built-in commands (e.g. for search).
     if (cmdparts.type == "builtin")
     {
       if (!execute_builtin_command(cmdparts.name))
@@ -1098,6 +1101,6 @@ void CommandUI::revalidate_edit_menu_items()
 
   // NOTE : using perform_from_main_thread causes a _grtm reference on the BaseEditor to to get lost in the process, 
   //        this causes the application to crash while attempting to create AutoUndoUpdate objects.
-  //        Changed to run_when_idle to fix this issue
+  //        Changed to run_once_when_idle to fix this issue
 } 
 
