@@ -27,6 +27,10 @@
 #include <grt/common.h>
 #include "base/string_utilities.h"
 
+#include "mforms/uistyle.h"
+#include "mforms/utilities.h"
+#include "mforms/checkbox.h"
+
 using namespace MySQL::Geometry;
 using namespace grtui;
 using namespace mforms;
@@ -201,6 +205,26 @@ db_mgmt_ConnectionRef DbConnectPanel::get_connection()
   return _connection->get_connection();
 }
 
+void DbConnectPanel::set_connection(const db_mgmt_ConnectionRef& conn)
+{
+  const grt::ListRef<db_mgmt_Connection> list(_connection->get_db_mgmt()->storedConns());
+
+  int count = 0;
+  grt::ListRef<db_mgmt_Connection>::const_iterator iter = list.begin();
+  grt::StringRef conn_host = conn->hostIdentifier();
+  for (;iter != list.end(); ++iter)
+  {
+    if (conn == (*iter))
+    {
+      _stored_connection_sel.set_selected(count + 1);
+      change_active_stored_conn();
+      break;
+    }
+
+    ++count;
+  }
+}
+
 void DbConnectPanel::set_enabled(bool flag)
 {
   _name_entry.set_enabled(flag);
