@@ -22,7 +22,7 @@
 @class MScintillaView;
 
 #include "sqlide/sql_editor_be.h"
-#import "string_utilities.h"
+#include "base/string_utilities.h"
 
 @interface MSQLEditorController : NSObject 
 {
