@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2009, 2011, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2009, 2012, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -28,11 +28,12 @@
 @class MTogglePane;
 @class MResultsetViewer;
 @class MVerticalLayoutView;
+@class MTabSwitcher;
 @class WBSplitView;
 @class WBSplitViewUnbrokenizerDelegate;
-@class WBOverviewPanel;
 @class WBMiniToolbar;
 @class GRTListDataSource;
+@class WBQueryTab;
 
 @interface WBSQLQueryPanel : WBBasePanel
 {
@@ -40,59 +41,57 @@
   IBOutlet WBSplitView* mWorkView;
   IBOutlet WBSplitViewUnbrokenizerDelegate* mSplitterDelegate;
   
+  IBOutlet NSSplitView* mSidebarSplit;
+  
+  IBOutlet WBMiniToolbar* mOutputToolbar;
   IBOutlet NSTabView* mOutputTabView;
-  IBOutlet NSSegmentedControl* mOutputSegmented;
+  IBOutlet NSPopUpButton* mOutputSelector;
   IBOutlet NSTextView* mTextOutput;
   
   IBOutlet NSTableView* mMessagesTable;
   IBOutlet NSTableView* mHistoryTable;
   IBOutlet NSTableView* mHistoryDetailsTable;
-  IBOutlet NSButton* mClearAllButton;
-
-  IBOutlet NSTextView* mOutputActionText;
-  IBOutlet NSTextView* mOutputResponseText;
 
   IBOutlet NSTabView* mUpperTabView;
-  IBOutlet NSTabView* mLowerTabView;
+  IBOutlet MTabSwitcher* mUpperTabSwitcher;
   
-  IBOutlet WBMiniToolbar* mOverviewToolbar;
-  IBOutlet WBMiniToolbar* mSnippetsToolbar;
   
-  IBOutlet NSTableView *snippetTable;
-  IBOutlet GRTListDataSource *snippetDS;
-  IBOutlet ScintillaView *mSnippetText;
-
-  NSView *mSidebarView;
+  NSView *mSchemataBarView;
 
   NSTimeInterval mLastClick;
   
   BOOL mQueryAreaOpen;
   BOOL mResultsAreaOpen;
+  BOOL mInitialized;
   
-  WBOverviewPanel *mOverview;
-
   NSMutableArray *mEditors;
   
-  Db_sql_editor::Ref mBackEnd;
+  SqlEditorForm::Ref mBackEnd;
   
   NSLock *mTextOutputLock;
   std::string mTextOutputBuffer;
   
-  BOOL sidebarHidden;
-  BOOL sidebarAtRight;
+  BOOL schematabarAtRight;
+  
+  CGFloat lastOutputAreaHeight;
   CGFloat lastSidebarWidth;
+  CGFloat lastSchematabarWidth;
 }
 
-- (IBAction)activateHistoryEntry:(id)sender;
+@property (readonly) SqlEditorForm::Ref backEnd;
+
 - (IBAction)clearOutput:(id)sender;
+- (IBAction)copyOutputEntry:(id)sender;
+- (IBAction)handleMenuAction:(id)sender;
+
+- (void)addEditor:(WBBasePanel*)editor;
 
 - (void)executeQuery:(id)sender currentStatementOnly: (bool) currentStatementOnly;
-- (id)initWithBE:(const Db_sql_editor::Ref&)be;
-- (Db_sql_editor::Ref)backEnd;
-- (MResultsetViewer*)selectedResultset;
+- (id)initWithBE:(const SqlEditorForm::Ref&)be;
 - (MSQLEditorController*)activeEditor;
-- (void)addEditorTab;
-- (void)addEditorTabWithBackEndIndex:(int)index;
+- (WBQueryTab*)activeQueryTab;
+- (WBQueryTab*)queryTabForBackEndIndex:(int)index;
+- (void)addSQLEditorTabWithBackEndIndex:(int)index;
 - (void)setRightSidebar:(BOOL)flag;
 - (void)flushOutputBuffer;
 @end
