@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2009, 2011, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2009, 2012, Oracle and/or its affiliates. All rights reserved.
  * 
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -12,7 +12,8 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */
+ *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ */
 
 #include "base/string_utilities.h"
 #import "WBMainController.h"
@@ -42,13 +43,16 @@
 #import "WBDiagramSizeController.h"
 #import "WBModelDiagramPanel.h"
 
-#include <mforms/mforms.h>
+#include "mforms/toolbar.h"
+#include "mforms/../dialogs/search_replace.h"
 
 #include "workbench/wb_module.h"
+#include "base/log.h"
 
 
 static GThread *mainthread= 0;
 
+DEFAULT_LOG_DOMAIN("Workbench")
 
 @implementation WBMainController
 
@@ -518,7 +522,8 @@ static bool validate_cut(wb::WBContextUI *wbui)
 
 //--------------------------------------------------------------------------------------------------
 
-static void call_closetab(WBMainWindow *mainWindow)
+// XXX deprecated, remove it eventually
+static void call_closetab_old(WBMainWindow *mainWindow)
 {
   if ([[mainWindow window] isKeyWindow])
   {
@@ -533,9 +538,35 @@ static void call_closetab(WBMainWindow *mainWindow)
   }
 }
 
+static void call_close_tab(WBMainWindow *mainWindow)
+{
+  if ([[mainWindow window] isKeyWindow])
+  {
+    id activePanel = [mainWindow activePanel];
+    bec::UIForm *form = [activePanel formBE];
+    if (form && form->get_form_context_name() == "home")
+      return;
+    
+    [mainWindow closePanel: activePanel];
+  }
+}
+
+static void call_close_editor(WBMainWindow *mainWindow)
+{
+  if ([[mainWindow window] isKeyWindow])
+  {
+    id activePanel = [mainWindow activePanel];
+    bec::UIForm *form = [activePanel formBE];
+    if (form && form->get_form_context_name() == "home")
+      return;
+
+    [activePanel closeActiveEditorTab];
+  }
+}
+
 //--------------------------------------------------------------------------------------------------
 
-static bool validate_closetab(WBMainWindow *mainWindow)
+static bool validate_closetab_old(WBMainWindow *mainWindow)
 {  
   id activePanel = [mainWindow activePanel];
   bec::UIForm *form = [activePanel formBE];
@@ -546,6 +577,24 @@ static bool validate_closetab(WBMainWindow *mainWindow)
   return activePanel != nil && [[mainWindow window] isKeyWindow];
 }
 
+static bool validate_close_tab(WBMainWindow *mainWindow)
+{  
+  id activePanel = [mainWindow activePanel];
+  bec::UIForm *form = [activePanel formBE];
+  if (form && form->get_form_context_name() == "home")
+    return false;
+  return activePanel != nil && [[mainWindow window] isKeyWindow];
+}
+
+static bool validate_close_editor(WBMainWindow *mainWindow)
+{  
+  id activePanel = [mainWindow activePanel];
+  bec::UIForm *form = [activePanel formBE];
+  if (form && form->get_form_context_name() == "home")
+    return false;
+  
+  return activePanel != nil && [[mainWindow window] isKeyWindow] && [activePanel respondsToSelector: @selector(closeActiveEditorTab)];
+}
 //--------------------------------------------------------------------------------------------------
 
 static void call_search(WBMainWindow *mainWindow)
@@ -598,7 +647,19 @@ static bool search_callback(const std::string &find, const std::string &replace_
 static void call_find_replace(wb::WBContextUI *wbui, bool no_replace)
 {
   id firstResponder = [[NSApp keyWindow] firstResponder];
-  if ([firstResponder isKindOfClass: [ScintillaView class]]
+  id superView = [firstResponder respondsToSelector: @selector(superview)] ? [firstResponder superview] : nil;
+  
+  if ([firstResponder isKindOfClass: [MScintillaView class]] && [[firstResponder delegate] respondsToSelector: @selector(showFindPanel:)])
+  {
+    [[firstResponder delegate] performSelector: @selector(showFindPanel:) withObject: firstResponder];
+    [firstResponder enableReplaceInFindPanel: !no_replace];
+  }
+  else if ([superView isKindOfClass: [MScintillaView class]] && [[superView delegate] respondsToSelector: @selector(showFindPanel:)])
+  {
+    [[superView delegate] performSelector: @selector(showFindPanel:) withObject: superView];
+    [superView enableReplaceInFindPanel: !no_replace];
+  }
+  else if ([firstResponder isKindOfClass: [ScintillaView class]]
       || [[firstResponder superview] isKindOfClass: [ScintillaView class]]
       || [firstResponder isKindOfClass: [NSTextView class]])
   {
@@ -635,11 +696,8 @@ static void call_find(WBMainWindow *mainWindow)
   // open the find dialog
   wb::WBContextUI *wbui = [mainWindow context];
   
-  if (wbui->get_active_context(true) != WB_CONTEXT_QUERY
-    && validate_find_replace(wbui))
-  {
+  if (validate_find_replace(wbui))
     call_find_replace(wbui, true);
-  }
   else
     [mainWindow focusSearchField:nil];
 }
@@ -734,16 +792,24 @@ static bool validate_redo(WBMainWindow *mainWindow)
   commands.push_back("wb.page_setup");
   //  commands.push_back("help_index");
   //  commands.push_back("help_version_check");
-  
+
   commands.push_back("wb.sidebarHide");
+
+  commands.push_back("wb.toggleSideBar");
+  commands.push_back("wb.toggleSchemataBar");
+  commands.push_back("wb.toggleOutputArea");
   
   commands.push_back("wb.next_tab");
   commands.push_back("wb.back_tab");
   
   _wbui->get_command_ui()->add_frontend_commands(commands);
 
-  _wbui->get_command_ui()->add_builtin_command("closetab", boost::bind(call_closetab, mainWindow), 
-                                               boost::bind(validate_closetab, mainWindow));
+  _wbui->get_command_ui()->add_builtin_command("closetab", boost::bind(call_closetab_old, mainWindow), 
+                                               boost::bind(validate_closetab_old, mainWindow));
+  _wbui->get_command_ui()->add_builtin_command("close_tab", boost::bind(call_close_tab, mainWindow), 
+                                               boost::bind(validate_close_tab, mainWindow));
+  _wbui->get_command_ui()->add_builtin_command("close_editor", boost::bind(call_close_editor, mainWindow), 
+                                               boost::bind(validate_close_editor, mainWindow));
   
   _wbui->get_command_ui()->add_builtin_command("searchbox", boost::bind(call_search, mainWindow));
                                                //boost::bind(validate_closetab), mainWindow));
@@ -839,7 +905,9 @@ static void flush_main_thread()
   else
   {
     cf_unswap_edit_menu();
-    (*_wb->get_ui()->get_command_ui()->signal_validate_edit_menu_items())();
+    bec::UIForm *form = _wb->get_active_main_form();
+    if (form && form->get_menubar())
+      form->get_menubar()->validate();
   }
 }
 
@@ -894,102 +962,6 @@ static NSString *applicationSupportFolder()
     wbcallbacks.lock_gui= boost::bind(windowLockGui, _1, mainWindow);
     wbcallbacks.quit_application= boost::bind(quitApplication);
       
-    _options= new wb::WBOptions();
-    _options->basedir = [[[NSBundle mainBundle] resourcePath] fileSystemRepresentation];
-    _options->struct_search_path = _options->basedir + "/grt";
-    _options->plugin_search_path = std::string([[[NSBundle mainBundle] builtInPlugInsPath] fileSystemRepresentation]);
-    _options->module_search_path = std::string([[[NSBundle mainBundle] builtInPlugInsPath] fileSystemRepresentation]);
-    _options->library_search_path = std::string([[[NSBundle mainBundle] resourcePath] fileSystemRepresentation]) + "/libraries";
-    _options->cdbc_driver_search_path = std::string([[[NSBundle mainBundle] privateFrameworksPath] fileSystemRepresentation]);
-    _options->user_data_dir= [[applicationSupportFolder() stringByAppendingString: @"/MySQL/Workbench"] fileSystemRepresentation];
- 
-    // handle some command line options
-    // --model=<modelfile>
-    // --query=<connection>
-    // --admin=<admin>
-    // --script=<scriptfile>
-    // --run=<script>
-    // --run-python=<script>
-    // --run-lua=<script>
-    // --quit-when-done
-    {
-      int argc= *_NSGetArgc();
-      char **argv= *_NSGetArgv();
-      for (int i= 1; i < argc; i++)
-      {
-        if (strncmp(argv[i], "--model=", strlen("--model=")) == 0
-            || strncmp(argv[i], "--query=", strlen("--query=")) == 0
-            || strncmp(argv[i], "--admin=", strlen("--admin=")) == 0
-            || strncmp(argv[i], "--script=", strlen("--script=")) == 0)
-        {
-          char *eq = strchr(argv[i], '=');
-          _options->open_at_startup_type= std::string(argv[i]+2, eq);
-          
-          if (strlen(eq+1) > 0)
-            _options->open_at_startup= eq+1;
-          else
-          {
-            NSLog(@"Missing argument for %s option", argv[i]);
-            exit(1);
-          }
-        }
-        else if (strncmp(argv[i], "--run=", strlen("--run=")) == 0)
-        {
-          char *eq = strchr(argv[i], '=');
-          if (strlen(eq+1) > 0)
-            _options->run_at_startup= eq+1;
-          else
-          {
-            NSLog(@"Missing argument for --run option");
-            exit(1);
-          }
-        }
-        else if (strncmp(argv[i], "--run-", strlen("--run-")) == 0)
-        {
-          char *eq = strchr(argv[i], '=');
-          if (!eq || strlen(eq+1) == 0)
-          {
-            NSLog(@"Missing argument for %s option", argv[i-1]);
-            exit(1);
-          }
-          
-          _options->run_language= std::string(argv[i]+strlen("--run-"), eq);
-          _options->run_at_startup= eq+1;
-        }        
-        else if (strcmp(argv[i], "--quit-when-done") == 0)
-          _options->quit_when_done= true;
-        else if (strcmp(argv[i], "--help") == 0)
-        {
-          NSLog(@"%s [<options>] [<model file>]", argv[0]);
-          NSLog(@"--model=<model file>             Open a model file at startup");
-          NSLog(@"--query=<connection name>        Starts a query tab to the named connection");
-          NSLog(@"--admin=<instance name>          Starts the administrator for the named instance");
-          NSLog(@"--script=<script file>           Executes the script file");
-          NSLog(@"--run=<script code>              Executes the script code, in the default scripting language");
-          NSLog(@"--run-python=<python code>       Executes the given Python code");
-          NSLog(@"--run-lua=<lua code>             Executes the given Lua code");
-          NSLog(@"--quit-when-done                 Quits Workbench when the script finished executing");
-          NSLog(@"--version                        Print version information and exit");
-          NSLog(@"--help                           Print this message and exit");
-          exit(0);
-        }
-        else if (strcmp(argv[i], "--version") == 0)
-        {
-          std::string release_type = APP_RELEASE_TYPE;
-          if (!release_type.empty())
-            release_type = " " + release_type;
-          
-#ifdef EDITION_SE
-          NSLog(@"MySQL Workbench SE %i.%i.%i%s rev %i", APP_MAJOR_NUMBER, APP_MINOR_NUMBER, APP_RELEASE_NUMBER, release_type.c_str(), APP_REVISION_NUMBER);
-#else
-          NSLog(@"MySQL Workbench CE %i.%i.%i%s rev %i", APP_MAJOR_NUMBER, APP_MINOR_NUMBER, APP_RELEASE_NUMBER, release_type.c_str(), APP_REVISION_NUMBER);
-#endif
-          exit(0);
-        }
-      }
-    }
-    //wboptions.open_at_startup = "";
-
     // add shipped python module search path to PYTHONPATH
     // also include mysql utilities module zip
     {
@@ -1021,12 +993,129 @@ static NSString *applicationSupportFolder()
   }
 }
 
+//--------------------------------------------------------------------------------------------------
+
+/**
+ * Instantiates the option class for the backend and parses the command line.
+ * @result Returns the passed log level to the caller or an empty string if none was given.
+ */
+- (std::string) setupOptionsAndParseCommandline
+{
+  _options= new wb::WBOptions();
+  _options->basedir = [[[NSBundle mainBundle] resourcePath] fileSystemRepresentation];
+  _options->struct_search_path = _options->basedir + "/grt";
+  _options->plugin_search_path = std::string([[[NSBundle mainBundle] builtInPlugInsPath] fileSystemRepresentation]);
+  _options->module_search_path = std::string([[[NSBundle mainBundle] builtInPlugInsPath] fileSystemRepresentation]);
+  _options->library_search_path = std::string([[[NSBundle mainBundle] resourcePath] fileSystemRepresentation]) + "/libraries";
+  _options->cdbc_driver_search_path = std::string([[[NSBundle mainBundle] privateFrameworksPath] fileSystemRepresentation]);
+  _options->user_data_dir= [[applicationSupportFolder() stringByAppendingString: @"/MySQL/Workbench"] fileSystemRepresentation];
+
+  // handle some command line options
+  // --model=<modelfile>
+  // --query=<connection>
+  // --admin=<admin>
+  // --script=<scriptfile>
+  // --run=<script>
+  // --run-python=<script>
+  // --run-lua=<script>
+  // --quit-when-done
+  // --log-level
+  std::string logLevel = "";
+  int argc= *_NSGetArgc();
+  char **argv= *_NSGetArgv();
+  for (int i= 1; i < argc; i++)
+  {
+    if (strncmp(argv[i], "--model=", strlen("--model=")) == 0
+        || strncmp(argv[i], "--query=", strlen("--query=")) == 0
+        || strncmp(argv[i], "--admin=", strlen("--admin=")) == 0
+        || strncmp(argv[i], "--script=", strlen("--script=")) == 0)
+    {
+      char *eq = strchr(argv[i], '=');
+      _options->open_at_startup_type= std::string(argv[i]+2, eq);
+      
+      if (strlen(eq+1) > 0)
+        _options->open_at_startup= eq+1;
+      else
+      {
+        NSLog(@"Missing argument for %s option", argv[i]);
+        exit(1);
+      }
+    }
+    else if (strncmp(argv[i], "--run=", strlen("--run=")) == 0)
+    {
+      char *eq = strchr(argv[i], '=');
+      if (strlen(eq+1) > 0)
+        _options->run_at_startup= eq+1;
+      else
+      {
+        NSLog(@"Missing argument for --run option");
+        exit(1);
+      }
+    }
+    else if (strncmp(argv[i], "--run-", strlen("--run-")) == 0)
+    {
+      char *eq = strchr(argv[i], '=');
+      if (!eq || strlen(eq+1) == 0)
+      {
+        NSLog(@"Missing argument for %s option", argv[i-1]);
+        exit(1);
+      }
+      
+      _options->run_language= std::string(argv[i]+strlen("--run-"), eq);
+      _options->run_at_startup= eq+1;
+    }        
+    else if (strcmp(argv[i], "--quit-when-done") == 0)
+      _options->quit_when_done= true;
+    else if (strncmp(argv[i], "--log-level=", strlen("--log-level=")) == 0)
+    {
+      char *eq = strchr(argv[i], '=');
+      if (!eq || strlen(eq+1) == 0)
+      {
+        NSLog(@"Missing argument for %s option", argv[i - 1]);
+        exit(1);
+      }
+      
+      logLevel = std::string(eq + 1);
+    }        
+    else if (strcmp(argv[i], "--help") == 0)
+    {
+      NSLog(@"%s [<options>] [<model file>]", argv[0]);
+      NSLog(@"--model=<model file>             Open a model file at startup");
+      NSLog(@"--query=<connection name>        Starts a query tab to the named connection");
+      NSLog(@"--admin=<instance name>          Starts the administrator for the named instance");
+      NSLog(@"--script=<script file>           Executes the script file");
+      NSLog(@"--run=<script code>              Executes the script code, in the default scripting language");
+      NSLog(@"--run-python=<python code>       Executes the given Python code");
+      NSLog(@"--run-lua=<lua code>             Executes the given Lua code");
+      NSLog(@"--quit-when-done                 Quits Workbench when the script finished executing");
+      NSLog(@"--log-level=<level>              Determines the amount of logging (error, warning, info, debug1/2/3)");
+      NSLog(@"--version                        Print version information and exit");
+      NSLog(@"--help                           Print this message and exit");
+      exit(0);
+    }
+    else if (strcmp(argv[i], "--version") == 0)
+    {
+#ifdef EDITION_SE
+      NSLog(@"MySQL Workbench SE %i.%i.%i rev %i %s", APP_MAJOR_NUMBER, APP_MINOR_NUMBER, APP_RELEASE_NUMBER, APP_REVISION_NUMBER, APP_RELEASE_TYPE);
+#else
+      NSLog(@"MySQL Workbench CE %i.%i.%i rev %i %s", APP_MAJOR_NUMBER, APP_MINOR_NUMBER, APP_RELEASE_NUMBER, APP_REVISION_NUMBER, APP_RELEASE_TYPE);
+#endif
+      exit(0);
+    }
+  }
+  return logLevel;
+}
+
+//--------------------------------------------------------------------------------------------------
+
 extern "C" {
   extern void mforms_cocoa_init();
   extern void mforms_cocoa_check();
 };
 static void init_mforms()
 {
+  log_debug("Initializing mforms\n");
+  
   extern void cf_grttreeview_init();
   extern void cf_tabview_init();
   static BOOL inited= NO;
@@ -1076,6 +1165,8 @@ static void init_mforms()
 
 - (void)dealloc
 {
+  log_info("Shutting down Workbench\n");
+
   [[NSNotificationCenter defaultCenter] removeObserver: self];
   
   [_editorWindows release];
@@ -1090,9 +1181,27 @@ static void init_mforms()
 
 
 - (void)awakeFromNib
-{  
+{ 
+  // Prepare the logger to be ready as first part.
+  base::Logger logger = base::Logger([[applicationSupportFolder() stringByAppendingString: @"/MySQL/Workbench"] fileSystemRepresentation]);
+#if DEBUG
+  std::string logLevel = "debug2";
+#else
+  std::string logLevel = "info";
+#endif
+  
+  char* environmentLevel = getenv("WB_LOG_LEVEL");
+  if (environmentLevel != NULL)
+    logLevel = environmentLevel;
+  
+  std::string passedLevel = [self setupOptionsAndParseCommandline];
+  if (passedLevel.size() > 0)
+    logLevel = passedLevel;
+  base::Logger::active_level(logLevel);
+  log_info("Starting up Workbench\n");
+  
   if (!mainWindow)
-  {    
+  {
     init_mforms();
     
     [NSApp setDelegate: self];
@@ -1145,15 +1254,20 @@ static void init_mforms()
 
 - (void)flushIdleTasks:(id)arg
 {
-  try
+  if (!_showingUnhandledException)
   {
-    _wb->flush_idle_tasks();
-  }
-  catch (const std::exception &exc)
-  {
-    NSRunAlertPanel(@"Unhandled Exception",
-                    [NSString stringWithFormat: @"An unhandled exception has occurred: %s", exc.what()],
-                    @"OK", nil, nil);
+    _showingUnhandledException = YES;
+    try
+    {
+      _wb->flush_idle_tasks();
+    }
+    catch (const std::exception &exc)
+    {
+      NSRunAlertPanel(@"Unhandled Exception",
+                      [NSString stringWithFormat: @"An unhandled exception has occurred: %s", exc.what()],
+                      @"OK", nil, nil);
+    }
+    _showingUnhandledException = NO;
   }
 }
 
