@@ -1,5 +1,5 @@
 /* 
- * Copyright © 2008, 2010, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2008, 2012, Oracle and/or its affiliates. All rights reserved.
  * 
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -58,8 +58,9 @@ static void DrawTiledImage(NSImage *image, NSRect rect, BOOL composite)
   if ((self= [super initWithFrame:rect]) != nil)
   {
     bgImage= [[NSImage imageNamed:@"background.png"] retain];
+    [bgImage setFlipped: YES];
     shadowImage= [[NSImage imageNamed:@"background_top_shadow.png"] retain];
-    [shadowImage setFlipped:YES];
+    [self setExpandSubviewsByDefault: NO];
   }
   return self;
 }
@@ -84,18 +85,19 @@ static void DrawTiledImage(NSImage *image, NSRect rect, BOOL composite)
 {
   if (shadowImage)
   {
-    rect= [self frame];
-    if (usedHeight < NSHeight(rect))
+    NSRect bounds= [self bounds];
+    NSRect rect = bounds;
+    if (minimumHeight < NSHeight(bounds))
     {
-      DrawTiledImage(bgImage, rect, NO);
+      DrawTiledImage(bgImage, NSMakeRect(0, 0, NSWidth(bounds), NSHeight(bounds) - minimumHeight), NO);
       
-      rect.size.height= usedHeight;
-      rect.origin.y= 0;
+      rect.size.height= minimumHeight;
+      rect.origin.y= NSHeight(bounds) - minimumHeight;
       [[NSColor whiteColor] set];
       NSRectFill(rect);
       
-      rect.origin.y= usedHeight;
       rect.size.height= [shadowImage size].height;
+      rect.origin.y= NSHeight(bounds) - minimumHeight - rect.size.height;
       DrawTiledImage(shadowImage, rect, YES);
     }
   }
@@ -327,7 +329,7 @@ static NSString *stringFromNodeId(const bec::NodeId &node)
             [group selectTabViewItemWithIdentifier: [[group tabViewItemAtIndex: [index intValue]] identifier]];
         }
       }
-    }  
+    }
 }
 
 
@@ -389,6 +391,22 @@ static NSString *stringFromNodeId(const bec::NodeId &node)
         
       default: break;
     }
+    
+    // find the container group of the refreshed node to request a size update
+    bec::NodeId parent;
+    
+    parent= _overview->get_parent(node);
+    while (parent.is_valid())
+    {
+      id container = [_itemContainers objectForKey: stringFromNodeId(parent)];
+      if ([container isKindOfClass: [WBOverviewGroupContainer class]])
+      {
+        [container tile];
+        break;
+      }
+      parent= _overview->get_parent(parent);
+    }
+    
     return;
   }
   catch (const std::exception &exc)
@@ -517,7 +535,7 @@ static int first_child_type(wb::OverviewBE *overview, const bec::NodeId &node)
       }
     }
   }
-  [_backgroundView relayout];
+  [_backgroundView tile];
 }
 
 @end
