@@ -5,7 +5,7 @@ using System.Runtime.Remoting.Channels;
 using System.Runtime.Remoting.Channels.Ipc;
 using System.Threading;
 
-namespace SingleInstanceApplication
+namespace MySQL.Workbench
 {
 	/// <summary>
 	/// Application Instance Manager
