@@ -287,6 +287,9 @@ namespace ScintillaNet
     {
       if (string.IsNullOrEmpty(lParam) || wParam == 0)
         return 0;
+
+      if (wParam == -1)
+        wParam = lParam.Length;
       char[] characters = lParam.ToCharArray(0, wParam);
       int byteCount = _encoding.GetByteCount(characters);
       Byte[] bytes = new Byte[byteCount];
@@ -1703,6 +1706,7 @@ namespace ScintillaNet
 
 		void INativeScintilla.SetIndentationGuides(bool view)
 		{
+      // ml: this is a bug, according to the docs view must be one of 4 values.
 			_ns.SendMessageDirect(Constants.SCI_SETINDENTATIONGUIDES, view, 0);
 		}
 
@@ -2986,7 +2990,9 @@ namespace ScintillaNet
 				TextModifiedEventArgs mea = new TextModifiedEventArgs
 					(
 					modType,
-					(modType | Constants.SC_PERFORMED_USER) > 0,
+          // ml: checking for a set bit needs logical and not or.
+          (modType & Constants.SC_PERFORMED_USER) > 0,
+					//(modType | Constants.SC_PERFORMED_USER) > 0,
 					scn.line,
 					scn.position,
 					scn.length,
