@@ -16,8 +16,8 @@ namespace ScintillaNet
 	[DefaultBindingProperty("Text"), DefaultProperty("Text"), DefaultEvent("DocumentChanged")]
 	public partial class Scintilla : System.Windows.Forms.Control, INativeScintilla, ISupportInitialize
 	{
-		public const string DefaultDllName = "SciLexer.dll";
-    private string _sciLexerDllName = "SciLexer.dll"; // this was null, but caused crashes in some systems -akk
+		public const string DefaultDllName = "Scintilla.dll"; //"SciLexer.dll";
+    private string _sciLexerDllName = "Scintilla.dll"; // this was null, but caused crashes in some systems -akk
 		
 
 		#region Property Bags
