@@ -1,12 +1,31 @@
+/* 
+ * Copyright (c) 2007, 2012, Oracle and/or its affiliates. All rights reserved.
+ *
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License as
+ * published by the Free Software Foundation; version 2 of the
+ * License.
+ * 
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+ * GNU General Public License for more details.
+ * 
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
+ * 02110-1301  USA
+ */
+
 using System;
 using System.Collections.Generic;
 using System.Drawing;
 using System.Windows.Forms;
 
 using MySQL.Controls;
+using MySQL.Forms;
 using MySQL.Grt;
 using MySQL.Grt.Db;
-using MySQL.Forms;
 
 namespace MySQL.GUI.Workbench.Plugins
 {
@@ -24,20 +43,13 @@ namespace MySQL.GUI.Workbench.Plugins
 
     private int refreshBlocked = 0;
 
-    /// <summary>
-    /// Used to implement the IWorkbenchDocument interface and support closing notification
-    /// </summary>
-    protected WorkbenchDocumentClosing onWorkbenchDocumentClosing = null;
-
-    protected BaseEditor editorBE;
+    private BaseEditor editorBE;
 
     private SplitContainer mainSplitContainer = null;
     private bool isEdictingLiveObjectCancelled = false;
-    //private ProgressBar applyingChangesProgressBar = null;
-    Button closeLiveEditorButton;
+    //Button closeLiveEditorButton;
     Button revertLiveObjectButton;
     Button applyLiveObjectButton;
-    private TextBox applyingChangesLogBox = null;
     public ObjectEditorView ContainerForm;
 
     #endregion
@@ -93,7 +105,7 @@ namespace MySQL.GUI.Workbench.Plugins
           dbObjEditorBE = editorBE as DBObjectEditorBE;
           if (sqlEditor.BE != null)
           {
-            MySQL.Grt.Db.Sql.Sql_editor sqlEditorBE = sqlEditor.BE;
+            MySQL.Grt.Db.Sql.SqlEditorWrapper sqlEditorBE = sqlEditor.BE;
             sqlEditor.BE = null;
             sqlEditorBE.Dispose();
           }
@@ -222,19 +234,24 @@ namespace MySQL.GUI.Workbench.Plugins
       }
     }
 
+    protected BaseEditor Backend
+    {
+      get { return editorBE; }
+      set
+      {
+        if (editorBE != null)
+          editorBE.Dispose();
+        editorBE = value;
+      }
+    }
+
     #endregion
 
     #region IWorkbenchDocument Interface
 
-    public override WorkbenchDocumentClosing OnWorkbenchDocumentClosing
+    public override UIForm BackendForm
     {
-      get { return onWorkbenchDocumentClosing; }
-      set { onWorkbenchDocumentClosing = value; }
-    }
-
-    public override UIForm BackendClass 
-    {
-      get { return editorBE; } 
+      get { return editorBE; }
     }
 
     public override void RefreshGUI(RefreshType refresh, String str, IntPtr ptr)
@@ -273,10 +290,6 @@ namespace MySQL.GUI.Workbench.Plugins
 
     public void ApplyChangesToLiveObject()
     {
-      editorBE.set_live_object_change_error_cb(ProcessLiveObjectChangeError);
-      editorBE.set_live_object_change_progress_cb(ProcessLiveObjectChangeProgress);
-      editorBE.set_live_object_change_statistics_cb(ProcessLiveObjectChangeStatistics);
-
       editorBE.apply_changes_to_live_object();
     }
 
@@ -308,6 +321,7 @@ namespace MySQL.GUI.Workbench.Plugins
             Panel liveObjectControlsPanel = new Panel();
             liveObjectControlsPanel.Parent = mainSplitContainer.Panel2;
             liveObjectControlsPanel.Dock = DockStyle.Fill;
+            /*
             {
               closeLiveEditorButton = new Button();
               closeLiveEditorButton.UseVisualStyleBackColor = true;
@@ -318,62 +332,43 @@ namespace MySQL.GUI.Workbench.Plugins
                 closeLiveEditorButton.Margin.Top);
               closeLiveEditorButton.Click += new EventHandler(closeLiveObjectEditorButton_Click);
             }
+             * */
             {
               revertLiveObjectButton = new Button();
               revertLiveObjectButton.UseVisualStyleBackColor = true;
               revertLiveObjectButton.Parent = liveObjectControlsPanel;
+              revertLiveObjectButton.FlatStyle = FlatStyle.System;
               revertLiveObjectButton.Text = "Revert";
               revertLiveObjectButton.Location = new Point(
-                closeLiveEditorButton.Location.X - closeLiveEditorButton.Margin.Left - revertLiveObjectButton.Width - revertLiveObjectButton.Margin.Right,
-                closeLiveEditorButton.Location.Y);
+                mainSplitContainer.Panel2.ClientSize.Width - revertLiveObjectButton.Width - revertLiveObjectButton.Margin.Right,
+                revertLiveObjectButton.Margin.Top);
               revertLiveObjectButton.Click += new EventHandler(revertChangesToLiveObjectButton_Click);
             }
             {
               applyLiveObjectButton = new Button();
               applyLiveObjectButton.UseVisualStyleBackColor = true;
               applyLiveObjectButton.Parent = liveObjectControlsPanel;
+              applyLiveObjectButton.FlatStyle = FlatStyle.System;
               applyLiveObjectButton.Text = "Apply";
               applyLiveObjectButton.Location = new Point(
                 revertLiveObjectButton.Location.X - revertLiveObjectButton.Margin.Left - applyLiveObjectButton.Width - applyLiveObjectButton.Margin.Right,
                 revertLiveObjectButton.Location.Y);
               applyLiveObjectButton.Click += new EventHandler(applyChangesToLiveObjectButton_Click);
             }
-            
-            /*
-            applyingChangesProgressBar = new ProgressBar();
-            {
-              applyingChangesProgressBar.Parent = liveObjectControlsPanel;
-              applyingChangesProgressBar.Location = new POINT(
-                revertLiveObjectButton.Location.X - revertLiveObjectButton.Margin.Left - applyingChangesProgressBar.Width - applyingChangesProgressBar.Margin.Right,
-                closeLiveEditorButton.Location.Y);
-            }
-            */
-            mainSplitContainer.Panel2MinSize = closeLiveEditorButton.Height + closeLiveEditorButton.Margin.Vertical;
+
+            mainSplitContainer.Panel2MinSize = revertLiveObjectButton.Height + revertLiveObjectButton.Margin.Vertical;
             mainSplitContainer.SplitterDistance = mainContainer.ClientSize.Height - mainSplitContainer.Panel2MinSize - mainSplitContainer.SplitterWidth;
 
-            closeLiveEditorButton.Anchor = revertLiveObjectButton.Anchor = applyLiveObjectButton.Anchor = /*applyingChangesProgressBar.Anchor =*/
-              AnchorStyles.Right | AnchorStyles.Bottom;
-
-            applyingChangesLogBox = new TextBox();
-            {
-              applyingChangesLogBox.Parent = liveObjectControlsPanel;
-              applyingChangesLogBox.ScrollBars = ScrollBars.Vertical;
-              applyingChangesLogBox.Multiline = true;
-              applyingChangesLogBox.Location = new Point(
-                applyingChangesLogBox.Margin.Left,
-                closeLiveEditorButton.Location.Y);
-              applyingChangesLogBox.Text = "DBMS feedback messages will go here upon applying changes.";
-            }
+            //closeLiveEditorButton.Anchor =
+            revertLiveObjectButton.Anchor =   AnchorStyles.Right | AnchorStyles.Bottom;
+            applyLiveObjectButton.Anchor = AnchorStyles.Right | AnchorStyles.Bottom;
+            /*applyingChangesProgressBar.Anchor =*/
           }
 
           mainSplitContainer.Parent = mainTabControl.Parent;
           mainTabControl.Parent = mainSplitContainer.Panel1;
 
-          //applyingChangesLogBox.Width = applyingChangesProgressBar.Location.X - applyingChangesProgressBar.Margin.Left - applyingChangesLogBox.Margin.Horizontal;
-          applyingChangesLogBox.Width = applyLiveObjectButton.Location.X - applyLiveObjectButton.Margin.Left - applyLiveObjectButton.Margin.Horizontal;
-          applyingChangesLogBox.Height = mainSplitContainer.Panel2.Height - applyingChangesLogBox.Margin.Vertical;
-          applyingChangesLogBox.Anchor = AnchorStyles.Bottom | AnchorStyles.Left | AnchorStyles.Right | AnchorStyles.Top;
-        }
+       }
         else
         {
           mainSplitContainer.Parent = null;
@@ -391,10 +386,6 @@ namespace MySQL.GUI.Workbench.Plugins
       if (null != sqlEd)
         sqlEd.sqlEditor.RunBackgroundAction(true);
 
-      applyingChangesLogBox.ForeColor = SystemColors.WindowText;
-      applyingChangesLogBox.Text = "";
-      //applyingChangesProgressBar.Value = applyingChangesProgressBar.Minimum;
-
       ApplyChangesToLiveObject();
     }
 
@@ -413,30 +404,6 @@ namespace MySQL.GUI.Workbench.Plugins
       Close(false);
     }
 
-    private int ProcessLiveObjectChangeError(Int64 errCode, String errMsg, String sql)
-    {
-      String errCodeMsg = (errCode == -1) ? "" : String.Format("SQL Error {0}: ", errCode);
-      String msg = String.Format("{0}{1}\r\nSQL: {2}\r\n\r\n", errCodeMsg, errMsg, sql.Trim());
-      applyingChangesLogBox.ForeColor = Color.Red;
-      applyingChangesLogBox.Text += msg;
-      return 0;
-    }
-
-    private int ProcessLiveObjectChangeProgress(float progress)
-    {
-      //applyingChangesProgressBar.Value =
-        //applyingChangesProgressBar.Minimum + (int)(progress * (applyingChangesProgressBar.Maximum - applyingChangesProgressBar.Minimum));
-      return 0;
-    }
-
-    private int ProcessLiveObjectChangeStatistics(int successCount, int failCount)
-    {
-      //applyingChangesProgressBar.Value = applyingChangesProgressBar.Maximum;
-      if (0 == failCount)
-        applyingChangesLogBox.Text = "Changes have been applied OK";
-      return 0;
-    }
-
     public override string TabText
     {
       set
@@ -485,9 +452,6 @@ namespace MySQL.GUI.Workbench.Plugins
         }
       }
 
-      if (onWorkbenchDocumentClosing != null)
-        onWorkbenchDocumentClosing(this, e);
-
       if (editorBE != null)
         editorBE.disable_auto_refresh();
     }
@@ -522,6 +486,9 @@ namespace MySQL.GUI.Workbench.Plugins
       insideRefreshFormData = true;
       try
       {
+        //This is not right to hack inside derived class, pick part of it containg tabs 
+        //and adjust something there leaving rest of controls unadjusted.
+        //There is no guariantee that derived editor will have anything named "mainTabControl" at all
         Control[] controls = Controls.Find("mainTabControl", true);
         if (controls.Length > 0)
         {
