@@ -21,6 +21,7 @@
 #define __BASE_LOG_IMPL_H__
 
 #include <string>
+#include <stdarg.h>
 #include "base/common.h"
 
 namespace base
@@ -32,44 +33,57 @@ class BASELIBRARY_PUBLIC_FUNC Logger
     // Note: When adding or changing integer id of log level mind that 
     // the numbers must be sequential from 0 to N. Also make sure that
     // log_impl.cpp:LevelText array is valid for new enum values
-    enum LogLevel {LogNone     = 0, 
-                   LogError    = 1, 
-                   LogWarning  = 2, 
-                   LogInfo     = 3, 
-                   LogDebug    = 4, 
-                   LogDebug2   = 5, 
-                   LogDebug3   = 6, 
-                   NumOfLevels = LogDebug3};
+    enum LogLevel {
+      LogNone     = 0, 
+      LogError    = 1, 
+      LogWarning  = 2, 
+      LogInfo     = 3, 
+      LogDebug    = 4, 
+      LogDebug2   = 5, 
+      LogDebug3   = 6, 
+      NumOfLevels = LogDebug3
+    };
+
     static void enable_level(const LogLevel level);
     static void disable_level(const LogLevel level);
     Logger(const std::string& dir); // Later logdir or set of log files can be passed
+    static void logv(const LogLevel level, const char* const domain, const char* format, va_list args);
     static void log(const LogLevel level, const char* const domain, const char* format, ...);
+    static void log_exc(const LogLevel level, const char* const domain, const char* msg, const std::exception &exc);
     static std::string get_state();
     static void set_state(const std::string& state);
+    static std::string log_filename();
+    static std::string log_dir();
+
+    static std::string active_level();
+    static bool active_level(const std::string& value);
   private:
     struct LoggerImpl;
-    static LoggerImpl*  _impl;
+    static LoggerImpl* _impl;
 };
 
 } // End of namespace
 
-#ifdef ENABLE_LOG
+#ifdef DEFAULT_LOG_DOMAIN
 
 #undef log_error
+#undef log_exception
 #undef log_warning
 #undef log_info
 #undef log_debug
 #undef log_debug2
+#undef log_debug3
 
-// log_domain is defined by ENABLE_LOg macro from base/log.h
-#define log_error(...) base::Logger::log(base::Logger::LogError, log_domain, __VA_ARGS__)
-#define log_warning(...) base::Logger::log(base::Logger::LogWarning, log_domain, __VA_ARGS__)
-#define log_info(...) base::Logger::log(base::Logger::LogInfo, log_domain, __VA_ARGS__)
-#define log_debug(...) base::Logger::log(base::Logger::LogDebug, log_domain, __VA_ARGS__)
-#define log_debug2(...) base::Logger::log(base::Logger::LogDebug2, log_domain, __VA_ARGS__)
-#define log_debug3(...) base::Logger::log(base::Logger::LogDebug3, log_domain, __VA_ARGS__)
+// log_domain is defined by DEFAULT_LOG_DOMAIN macro from base/log.h
+#define log_error(...) base::Logger::log(base::Logger::LogError, default_log_domain, __VA_ARGS__)
+#define log_exception(msg, exc) base::Logger::log_exc(base::Logger::LogError, default_log_domain, msg, exc)
+#define log_warning(...) base::Logger::log(base::Logger::LogWarning, default_log_domain, __VA_ARGS__)
+#define log_info(...) base::Logger::log(base::Logger::LogInfo, default_log_domain, __VA_ARGS__)
+#define log_debug(...) base::Logger::log(base::Logger::LogDebug, default_log_domain, __VA_ARGS__)
+#define log_debug2(...) base::Logger::log(base::Logger::LogDebug2, default_log_domain, __VA_ARGS__)
+#define log_debug3(...) base::Logger::log(base::Logger::LogDebug3, default_log_domain, __VA_ARGS__)
 
-#endif // ifdef ENABLE_LOG
+#endif // ifdef DEFAULT_LOG_DOMAIN
 
 #endif
 
