@@ -20,21 +20,26 @@
 #import "MFToolBar.h"
 #import "MFBase.h"
 #include "mforms/toolbar.h"
-#include "mforms/mforms.h"
+#import "MFMForms.h"
 
-static const float SMALL_TOOLBAR_HEIGHT = 20;
-static const float SMALL_TOOLBAR_XPADDING = 8;
-static const float SMALL_TOOLBAR_YPADDING = 1;
-static const float SMALL_TOOLBAR_ITEM_WIDTH = SMALL_TOOLBAR_HEIGHT - 2*SMALL_TOOLBAR_YPADDING;
-static const float SMALL_TOOLBAR_SPACING = 4;
 
+static struct
+{
+  const float height;
+  const float xpadding;
+  const float ypadding;
+  const float item_width;
+  const float spacing;
+} layout_info[] = 
+{
+  {36, 12, 5, 36 - 2 * 7, 7}, // Main
+  {26, 12, 2, 25 - 2 * 7, 7}, // Secondary
+  {36, 12, 7, 36 - 2 * 7, 7}, // ToolPicker
+  {20, 8, 1, 20-2*1, 4}, // Options
+  {26, 4, 3, 26-2*3, 2} // Palette
+};
 
 static const float EMPTY_TOOLBAR_HEIGHT = 6;
-static const float NORMAL_TOOLBAR_HEIGHT = 36;
-static const float NORMAL_TOOLBAR_XPADDING = 12;
-static const float NORMAL_TOOLBAR_YPADDING = 7;
-static const float NORMAL_TOOLBAR_ITEM_WIDTH = NORMAL_TOOLBAR_HEIGHT - 2*NORMAL_TOOLBAR_YPADDING;
-static const float NORMAL_TOOLBAR_SPACING = 7;
 
 using namespace mforms;
 
@@ -48,6 +53,7 @@ using namespace mforms;
 - (id)initWithItemObject:(ToolBarItem*)item;
 - (ToolBarItemType)itemType;
 - (void)setHorizontal:(BOOL)flag;
+- (ToolBarItem*)toolBarItem;
 @end
 
 
@@ -55,7 +61,7 @@ using namespace mforms;
 
 - (id)initWithItemObject:(ToolBarItem*)item
 {
-  self = [super initWithFrame: NSMakeRect(0, 0, 1, NORMAL_TOOLBAR_ITEM_WIDTH)];
+  self = [super initWithFrame: NSMakeRect(0, 0, 1, layout_info[0].height)];
   if (self)
   {
     mOwner = item;
@@ -64,6 +70,11 @@ using namespace mforms;
   return self;
 }
 
+- (ToolBarItem*)toolBarItem
+{
+  return mOwner;
+}
+
 - (void)setHorizontal:(BOOL)flag
 {
   mHorizontal= flag;
@@ -86,26 +97,36 @@ using namespace mforms;
 {
   if (mOwner->get_type() == SeparatorItem)
   {
-    NSBezierPath *vdotted;
+    MFToolBarImpl *toolbar = (MFToolBarImpl*)[self superview];
     
-    CGFloat pattern[] = {1.0, 2.0};
-    vdotted = [[NSBezierPath bezierPath] retain];
-    [vdotted setLineWidth: 1.0];
-    [vdotted setLineDash: pattern count: 2 phase: 0.0];
-    
-    if (mHorizontal)
+    if ([toolbar type] == SecondaryToolBar)
     {
-      [vdotted moveToPoint: NSMakePoint(0, 0)];
-      [vdotted lineToPoint: NSMakePoint(NSWidth([self frame]), 0)];
+      [[NSColor colorWithDeviceWhite: 140/255.0 alpha: 1.0] set];
+      [NSBezierPath strokeLineFromPoint: NSMakePoint(NSMinX(rect), 2) toPoint: NSMakePoint(NSMinX(rect), NSHeight(rect)-1)];
     }
     else
     {
-      [vdotted moveToPoint: NSMakePoint(0, 0)];
-      [vdotted lineToPoint: NSMakePoint(0, NSHeight([self frame]))];
+      NSBezierPath *vdotted;
+      
+      CGFloat pattern[] = {1.0, 2.0};
+      vdotted = [[NSBezierPath bezierPath] retain];
+      [vdotted setLineWidth: 1.0];
+      [vdotted setLineDash: pattern count: 2 phase: 0.0];
+      
+      if (mHorizontal)
+      {
+        [vdotted moveToPoint: NSMakePoint(0, 0)];
+        [vdotted lineToPoint: NSMakePoint(NSWidth([self frame]), 0)];
+      }
+      else
+      {
+        [vdotted moveToPoint: NSMakePoint(0, 0)];
+        [vdotted lineToPoint: NSMakePoint(0, NSHeight([self frame]))];
+      }
+      
+      [[NSColor darkGrayColor] set];
+      [vdotted stroke];
     }
-    
-    [[NSColor darkGrayColor] set];
-    [vdotted stroke];
   }
 }
 
@@ -119,6 +140,7 @@ using namespace mforms;
   BOOL mToolPicker;
 }
 - (id)initWithItemObject:(ToolBarItem*)item;
+- (ToolBarItem*)toolBarItem;
 @end
 
 
@@ -126,22 +148,43 @@ using namespace mforms;
 
 - (id)initWithItemObject:(ToolBarItem*)item
 {
-  self = [super initWithFrame: NSMakeRect(0, 0, NORMAL_TOOLBAR_ITEM_WIDTH, NORMAL_TOOLBAR_ITEM_WIDTH)];
+  self = [super initWithFrame: NSMakeRect(0, 0, layout_info[0].item_width, layout_info[0].item_width)];
   if (self)
   {
     mOwner = item;
     mOwner->set_data(self);
-    if (item->get_type() == ToggleItem)
-      [self setButtonType: NSToggleButton];
-    [self setImage: [NSImage imageNamed: @"NSInfo"]];
+    switch (item->get_type())
+    {
+      case ToggleItem:
+        [self setButtonType: NSToggleButton];
+        [[self cell] setHighlightsBy: NSChangeBackgroundCellMask];
+        break;
+      case TextActionItem:
+        [self setImagePosition: NSNoImage];
+        break;
+      case SegmentedToggleItem:
+        [self setButtonType: NSToggleButton];
+        [self setImagePosition: NSImageOnly];
+        [[self cell] setHighlightsBy: NSChangeBackgroundCellMask];
+        break;
+      default:
+        [self setImage: [NSImage imageNamed: @"NSInfo"]];
+        [[self cell] setHighlightsBy: NSChangeBackgroundCellMask];
+        //[[self cell] setImageScaling: NSImageScaleProportionallyDown];
+        break;
+    }
     [self setBordered: NO];
-    [[self cell] setImageScaling: NSImageScaleProportionallyDown];
     [self setTarget: self];
     [self setAction: @selector(perform:)];
   }
   return self;
 }
 
+- (ToolBarItem*)toolBarItem
+{
+  return mOwner;
+}
+
 - (void)perform:(id)sender
 {
   mOwner->callback();
@@ -157,6 +200,11 @@ using namespace mforms;
     [self setBordered: value == NSOnState];
 }
 
+- (void)setImage:(NSImage *)image
+{
+  [super setImage: image];
+  [self setFrameSize: [image size]];
+}
 
 - (void)viewDidMoveToSuperview
 {
@@ -171,7 +219,8 @@ using namespace mforms;
   }
   else
   {
-    [[self cell] setHighlightsBy: NSContentsCellMask];
+    if (mOwner->get_type() == ActionItem)
+      [[self cell] setHighlightsBy: NSContentsCellMask];
   }
 }
 
@@ -185,22 +234,29 @@ using namespace mforms;
   ToolBarItem *mOwner;
 }
 - (id)initWithItemObject:(ToolBarItem*)item;
+- (ToolBarItem*)toolBarItem;
 @end
 
 @implementation MFToolBarLabelItemImpl
 - (id)initWithItemObject:(ToolBarItem*)item
 {
-  self = [super initWithFrame: NSMakeRect(0, 0, 200, NORMAL_TOOLBAR_ITEM_WIDTH)];
+  self = [super initWithFrame: NSMakeRect(0, 0, 200, layout_info[0].item_width)];
   if (self)
   {
     mOwner = item;
     mOwner->set_data(self);
     [self setBordered: NO];
+    [self setEditable: NO];
     [self setDrawsBackground: NO];
   }
   return self;
 }
 
+- (ToolBarItem*)toolBarItem
+{
+  return mOwner;
+}
+
 - (void) viewDidMoveToSuperview
 {
   switch ([(MFToolBarImpl*)[self superview] type])
@@ -227,12 +283,13 @@ using namespace mforms;
   ToolBarItem *mOwner;
 }
 - (id)initWithItemObject:(ToolBarItem*)item;
+- (ToolBarItem*)toolBarItem;
 @end
 
 @implementation MFToolBarImageItemImpl
 - (id)initWithItemObject:(ToolBarItem*)item
 {
-  self = [super initWithFrame: NSMakeRect(0, 0, NORMAL_TOOLBAR_ITEM_WIDTH, NORMAL_TOOLBAR_ITEM_WIDTH)];
+  self = [super initWithFrame: NSMakeRect(0, 0, layout_info[0].item_width, layout_info[0].item_width)];
   if (self)
   {
     mOwner = item;
@@ -243,6 +300,11 @@ using namespace mforms;
   return self;
 }
 
+- (ToolBarItem*)toolBarItem
+{
+  return mOwner;
+}
+
 - (void)setImage:(NSImage*)image
 {
   [super setImage: image];
@@ -258,12 +320,13 @@ using namespace mforms;
   ToolBarItem *mOwner;
 }
 - (id)initWithItemObject:(ToolBarItem*)item;
+- (ToolBarItem*)toolBarItem;
 @end
 
 @implementation MFToolBarSearchItemImpl
 - (id)initWithItemObject:(ToolBarItem*)item
 {
-  self = [super initWithFrame: NSMakeRect(0, 0, 200, NORMAL_TOOLBAR_ITEM_WIDTH)];
+  self = [super initWithFrame: NSMakeRect(0, 0, 200, layout_info[0].item_width)];
   if (self)
   {
     mOwner = item;
@@ -276,6 +339,11 @@ using namespace mforms;
   return self;
 }
 
+- (ToolBarItem*)toolBarItem
+{
+  return mOwner;
+}
+
 - (void)perform:(id)sender
 {
   mOwner->callback();
@@ -289,12 +357,13 @@ using namespace mforms;
   ToolBarItem *mOwner;
 }
 - (id)initWithItemObject:(ToolBarItem*)item;
+- (ToolBarItem*)toolBarItem;
 @end
 
 @implementation MFToolBarSelectorItemImpl
 - (id)initWithItemObject:(ToolBarItem*)item
 {
-  self = [super initWithFrame: NSMakeRect(0, 0, 200, NORMAL_TOOLBAR_ITEM_WIDTH)];
+  self = [super initWithFrame: NSMakeRect(0, 0, 200, layout_info[0].item_width)];
   if (self)
   {
     mOwner = item;
@@ -307,6 +376,11 @@ using namespace mforms;
   return self;
 }
 
+- (ToolBarItem*)toolBarItem
+{
+  return mOwner;
+}
+
 - (void) viewDidMoveToSuperview
 {
   switch ([(MFToolBarImpl*)[self superview] type])
@@ -342,8 +416,6 @@ using namespace mforms;
 
 @end
 
-
-
 //--------------------------------------------------------------------------------------------------
 
 @implementation MFToolBarImpl
@@ -355,14 +427,22 @@ using namespace mforms;
     case MainToolBar:
       self = [super initWithFrame:NSMakeRect(0, 0, 100, EMPTY_TOOLBAR_HEIGHT)];
       break;
+
+    case SecondaryToolBar:
+      self = [super initWithFrame:NSMakeRect(0, 0, 100, layout_info[SecondaryToolBar].height)];
+      break;
       
-    case ToolPickerToolBar:
-      self = [super initWithFrame:NSMakeRect(0, 0, NORMAL_TOOLBAR_HEIGHT, 100)];
+    case ToolPickerToolBar: // this is vertical
+      self = [super initWithFrame:NSMakeRect(0, 0, layout_info[ToolPickerToolBar].height, 100)];
       break;
       
     case OptionsToolBar:
-      self = [super initWithFrame:NSMakeRect(0, 0, 100, SMALL_TOOLBAR_HEIGHT)];
+      self = [super initWithFrame:NSMakeRect(0, 0, 100, layout_info[OptionsToolBar].height)];
       break;
+      
+    case PaletteToolBar:
+      self = [super initWithFrame:NSMakeRect(0, 0, 100, layout_info[PaletteToolBar].height)];
+      break;      
   }
       
   if (self)
@@ -385,26 +465,37 @@ using namespace mforms;
 
 - (NSSize)minimumSize
 {
-  switch (mOwner->get_type()) 
+  switch (mOwner->get_type())
   {
     case MainToolBar:
       if ([[self subviews] count] == 0)
         return NSMakeSize(100, EMPTY_TOOLBAR_HEIGHT);
       else
-        return NSMakeSize(100, NORMAL_TOOLBAR_HEIGHT);
-      
+        return NSMakeSize(100, layout_info[MainToolBar].height);
+
+    case SecondaryToolBar:
+      return NSMakeSize(100, layout_info[SecondaryToolBar].height);
+
     case ToolPickerToolBar:
-      return NSMakeSize(NORMAL_TOOLBAR_HEIGHT, 100);
+      return NSMakeSize(layout_info[ToolPickerToolBar].height, 100);
       
     case OptionsToolBar:
-      return NSMakeSize(100, SMALL_TOOLBAR_HEIGHT);
+      return NSMakeSize(100, layout_info[OptionsToolBar].height);
       
-    default:
-      return NSZeroSize;
+    case PaletteToolBar:
+      return NSMakeSize(100, layout_info[PaletteToolBar].height);
   }
+  return NSMakeSize(1, 1);
 }
 
 
+- (BOOL)expandsOnLayoutVertically:(BOOL)vertically
+{
+  if (vertically)
+    return NO;
+  return YES;
+}
+
 - (void)setTag:(NSInteger)tag
 {
   mTag = tag;
@@ -426,18 +517,20 @@ using namespace mforms;
 
   if ([[self subviews] count] == 0)
     wasEmpty = YES;
-  
-  if (index >= [[self subviews] count])
-    [self addSubview: item];
-  else
-    [self addSubview: item positioned: NSWindowBelow relativeTo: [[self subviews] objectAtIndex: index]];
 
+  NSView *view = item;
+  if (index >= [[self subviews] count])
+    [self addSubview: view];
+  else
+    [self addSubview: view positioned: NSWindowBelow relativeTo: [[self subviews] objectAtIndex: index]];
   if (wasEmpty)
   {
+    // main toolbar starts with a reduced height if it's empty. so when something is added, it has to
+    // be resized to the proper height
     if (mOwner->get_type() == MainToolBar)
     {
       NSRect rect = [self frame];
-      rect.size.height = NORMAL_TOOLBAR_HEIGHT;
+      rect.size.height = layout_info[MainToolBar].height;
       [self setFrame: rect];
       [super subviewMinimumSizeChanged];
     }
@@ -466,7 +559,7 @@ using namespace mforms;
 
 - (void)resizeSubviewsWithOldSize:(NSSize)oldBoundsSize
 {
-  NSRect rect = [self frame];
+  NSRect rect = [self bounds];
   
   if (mOwner->get_type() == ToolPickerToolBar)
   {
@@ -474,8 +567,8 @@ using namespace mforms;
     float xpadding = 1;
     float ypadding = 4;
     float x = NSMinX(rect) + xpadding;
-    float y = NSMinY(rect) + ypadding;    
-    float w = NORMAL_TOOLBAR_HEIGHT - 2 * ypadding;
+    float y = NSMinY(rect) + ypadding;
+    float w = layout_info[ToolPickerToolBar].height - 2 * ypadding;
     
     int item_count = 0;
     int expander_count = 0;
@@ -498,7 +591,6 @@ using namespace mforms;
       r.size.width = w;
       if ([item isKindOfClass: [MFToolBarActionItemImpl class]])
         r.size.height = w;
-
       if ([item isKindOfClass: [MFToolBarSeparatorImpl class]] && [item itemType] == ExpanderItem)
       {
         r.size.height = (NSHeight(rect) - total_height) / expander_count;
@@ -513,31 +605,44 @@ using namespace mforms;
   }
   else
   {
-    float spacing = NORMAL_TOOLBAR_SPACING;
-    float xpadding = mOwner->get_type() == MainToolBar ? NORMAL_TOOLBAR_XPADDING : SMALL_TOOLBAR_XPADDING;
-    float ypadding = mOwner->get_type() == MainToolBar ? NORMAL_TOOLBAR_YPADDING : SMALL_TOOLBAR_YPADDING;
+    ToolBarType type = mOwner->get_type();
+    float spacing = layout_info[type].spacing;
+    float xpadding = layout_info[type].xpadding;
+    float ypadding = layout_info[type].ypadding;
     float x = NSMinX(rect) + xpadding;
     float y = NSMinY(rect) + ypadding;    
-    float h = (mOwner->get_type() == MainToolBar ? NORMAL_TOOLBAR_HEIGHT : SMALL_TOOLBAR_HEIGHT) - 2 * ypadding;
+    float h = layout_info[type].height - 2 * ypadding;
     
+    int spaced_item_count = 0;
     int item_count = 0;
     int expander_count = 0;
     float total_width = xpadding*2;
     
     for (id item in [self subviews])
     {
-      total_width += NSWidth([item frame]);
       if ([item isKindOfClass: [MFToolBarSeparatorImpl class]] && [item itemType] == ExpanderItem)
         expander_count++;
       else
+      {
+        mforms::ToolBarItem *titem = [item toolBarItem];
+        if (titem && titem->get_expandable())
+          expander_count++;
+        else
+          total_width += NSWidth([item frame]);
+        if (titem && titem->get_type() != SegmentedToggleItem)
+          spaced_item_count++;
         item_count++;
+      }
     }
-    total_width += spacing * item_count;
+    total_width += spacing * spaced_item_count;
     
     for (id item in [self subviews])
     {
       NSRect r = [item frame];
       r.origin = NSMakePoint(x, y);
+      mforms::ToolBarItem *titem = [item toolBarItem];
+      if (titem && titem->get_expandable())
+        r.size.width = (NSWidth(rect) - total_width) / expander_count;
       if ([item isKindOfClass: [MFToolBarSeparatorImpl class]] && [item itemType] == ExpanderItem)
       {
         r.size.height = h;
@@ -560,7 +665,11 @@ using namespace mforms;
         r.size.height = h;
         [item setFrame: r];
       }
-      x += NSWidth(r) + spacing;
+      
+      if (titem && titem->get_type() == SegmentedToggleItem)
+        x += NSWidth(r);
+      else
+        x += NSWidth(r) + spacing;
     }
   }
 }
@@ -574,40 +683,47 @@ using namespace mforms;
 - (void)drawRect:(NSRect)rect
 {
   NSGradient *grad;
-  
-  if (mOwner->get_type() != ToolPickerToolBar)
+  rect = [self bounds];
+
+  switch (mOwner->get_type())
   {
-    if (mOwner->get_type() == OptionsToolBar)
-    {
-      grad = [[NSGradient alloc] initWithStartingColor: [NSColor colorWithDeviceWhite: 243/255.0 alpha:1.0] 
-                                           endingColor: [NSColor colorWithDeviceWhite: 230/255.0 alpha:1.0]];
-    }
-    else
-    {
-      if ([[self window] isMainWindow])
-        grad = [[NSGradient alloc] initWithStartingColor: [NSColor colorWithDeviceWhite: 206/255.0 alpha:1.0] 
-                                             endingColor: [NSColor colorWithDeviceWhite: 188/255.0 alpha:1.0]];
+    case ToolPickerToolBar:
+    case PaletteToolBar:
+    case SecondaryToolBar:
+      [[self backgroundColor] set];
+      NSRectFill(rect);
+      [[NSColor colorWithDeviceWhite: 202/255.0 alpha: 1.0] set];
+      [NSBezierPath setDefaultLineWidth: 0.0];
+      [NSBezierPath strokeLineFromPoint: NSMakePoint(0, NSHeight(rect)-0.5) toPoint: NSMakePoint(NSWidth(rect), NSHeight(rect)-0.5)];
+      break;
+
+    default:
+      if (mOwner->get_type() == OptionsToolBar)
+      {
+        grad = [[NSGradient alloc] initWithStartingColor: [NSColor colorWithDeviceWhite: 243/255.0 alpha:1.0] 
+                                             endingColor: [NSColor colorWithDeviceWhite: 230/255.0 alpha:1.0]];
+      }
       else
-        grad = [[NSGradient alloc] initWithStartingColor: [NSColor colorWithDeviceWhite: 233/255.0 alpha:1.0] 
-                                             endingColor: [NSColor colorWithDeviceWhite: 216/255.0 alpha:1.0]];
-    }
-    rect = [self frame];
-    [grad drawInRect: rect angle: 90.0];
-    [grad release];
-    
-    [[NSColor colorWithDeviceWhite: 202/255.0 alpha: 1.0] set];
-    [NSBezierPath setDefaultLineWidth: 0.0];
-    if (mOwner->get_type() != OptionsToolBar && [[self window] isMainWindow])
-    {
-      [NSBezierPath strokeLineFromPoint: NSMakePoint(0, NSHeight(rect)-1.5) toPoint: NSMakePoint(NSWidth(rect), NSHeight(rect)-1.5)];
-      [[NSColor colorWithDeviceWhite: 117/255.0 alpha: 1.0] set];
-    }
-    [NSBezierPath strokeLineFromPoint: NSMakePoint(0, NSHeight(rect)-0.5) toPoint: NSMakePoint(NSWidth(rect), NSHeight(rect)-0.5)];
-  }
-  else
-  {
-    [[self backgroundColor] set];
-    NSRectFill(rect);
+      {
+        if ([[self window] isMainWindow])
+          grad = [[NSGradient alloc] initWithStartingColor: [NSColor colorWithDeviceWhite: 206/255.0 alpha:1.0] 
+                                               endingColor: [NSColor colorWithDeviceWhite: 188/255.0 alpha:1.0]];
+        else
+          grad = [[NSGradient alloc] initWithStartingColor: [NSColor colorWithDeviceWhite: 233/255.0 alpha:1.0] 
+                                               endingColor: [NSColor colorWithDeviceWhite: 216/255.0 alpha:1.0]];
+      }
+      [grad drawInRect: rect angle: 90.0];
+      [grad release];
+      
+      [[NSColor colorWithDeviceWhite: 202/255.0 alpha: 1.0] set];
+      [NSBezierPath setDefaultLineWidth: 0.0];
+      if (mOwner->get_type() != OptionsToolBar && [[self window] isMainWindow])
+      {
+        [NSBezierPath strokeLineFromPoint: NSMakePoint(0, NSHeight(rect)-1.5) toPoint: NSMakePoint(NSWidth(rect), NSHeight(rect)-1.5)];
+        [[NSColor colorWithDeviceWhite: 117/255.0 alpha: 1.0] set];
+      }
+      [NSBezierPath strokeLineFromPoint: NSMakePoint(0, NSHeight(rect)-0.5) toPoint: NSMakePoint(NSWidth(rect), NSHeight(rect)-0.5)];
+      break;
   }
 }
 
@@ -626,6 +742,12 @@ using namespace mforms;
   return NO;
 }
 
+
+- (void)destroy
+{
+  [self removeFromSuperview];
+}
+
 @end
 
 
@@ -654,6 +776,8 @@ static bool create_tool_item(ToolBarItem *item, ToolBarItemType type)
   {
     case ActionItem:
     case ToggleItem:
+    case TextActionItem:
+    case SegmentedToggleItem:
       [[[MFToolBarActionItemImpl alloc] initWithItemObject: item] autorelease];
       break;
       
@@ -665,22 +789,22 @@ static bool create_tool_item(ToolBarItem *item, ToolBarItemType type)
     case SearchFieldItem:
       [[[MFToolBarSearchItemImpl alloc] initWithItemObject: item] autorelease];
       break;
-      
+
     case SelectorItem:
       [[[MFToolBarSelectorItemImpl alloc] initWithItemObject: item] autorelease];
       break;
-      
+
     case ColorSelectorItem:
       [[[MFToolBarSelectorItemImpl alloc] initWithItemObject: item] autorelease];
       break;
-      
+
     case LabelItem:
       [[[MFToolBarLabelItemImpl alloc] initWithItemObject: item] autorelease];
       break;
-      
+
     case ImageBoxItem:
       [[[MFToolBarImageItemImpl alloc] initWithItemObject: item] autorelease];
-      break;      
+      break;
   }
   
   return true;
@@ -709,13 +833,19 @@ static void set_item_alt_icon(ToolBarItem *item, const std::string &image)
 static void set_item_text(ToolBarItem *item, const std::string &text)
 {
   id tbitem = item->get_data();
-  [tbitem setStringValue: wrap_nsstring(text)];
+  if ([tbitem isKindOfClass: [NSButton class]])
+    [tbitem setTitle: wrap_nsstring(text)];
+  else
+    [tbitem setStringValue: wrap_nsstring(text)];
 }
 
 static std::string get_item_text(ToolBarItem *item)
 {
   id tbitem = item->get_data();
-  return [[tbitem stringValue] UTF8String] ?: "";
+  if ([tbitem isKindOfClass: [NSButton class]])
+    return [[tbitem title] UTF8String];
+  else    
+    return [[tbitem stringValue] UTF8String] ?: "";
 }
 
 static void set_item_enabled(ToolBarItem *item, bool flag)
