@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2009, 2010, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2009, 2012, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -38,6 +38,10 @@
 #import "MTextImageCell.h"
 #import "GRTTreeDataSource.h"
 #import "MResultsetViewer.h"
+#import "MTabSwitcher.h"
+#import "MColoredView.h"
+
+#import "WBTabItem.h" // needed for WBCustomTabItemView
 
 #import "DbPrivilegeEditorTab.h"
 #include "base/string_utilities.h"
@@ -71,6 +75,9 @@ static void call_refresh(DbMysqlTableEditor* theEditor)
     NSString* name = [NSString stringWithCPPString: mBackEnd->get_name()];
     [mTableName setStringValue: name];
     
+    [mSchemaName setEnabled: !mBackEnd->is_editing_live_object()];
+    [mSchemaName setStringValue: [NSString stringWithCPPString: mBackEnd->get_schema_name()]];
+
     NSString* collation = [NSString stringWithCPPString: mBackEnd->get_table_option_by_name("CHARACTER SET - COLLATE")];
     if ([collation isEqualToString: @" - "] || [collation length] == 0) {
       collation = @"Schema Default";
@@ -95,7 +102,7 @@ static void call_refresh(DbMysqlTableEditor* theEditor)
 
 
 // Set up values for the Columns tab.
-- (void) refreshTableEditorGUIColumnsTab;
+- (void)refreshTableEditorGUIColumnsTab;
 {
   [mColumnsTable reloadData];
   
@@ -114,66 +121,38 @@ static void call_refresh(DbMysqlTableEditor* theEditor)
     NSString* columnName = [mColumnsDataSource objectValueForValueIndex: bec::TableColumnsListBE::Name
                                                                     row: rowIndex];
     [mColumnsDetailsBox setTitle: [NSString stringWithFormat: @"Column details '%@'", columnName]];
+    [mColumnsName setStringValue: columnName];
+    [mColumnsType setStringValue: [mColumnsDataSource objectValueForValueIndex: bec::TableColumnsListBE::Type 
+                                                                           row: rowIndex]];
+    [mColumnsDefault setStringValue: [mColumnsDataSource objectValueForValueIndex: bec::TableColumnsListBE::Default
+                                                                              row: rowIndex]];
     
     NSString* comments = [mColumnsDataSource objectValueForValueIndex: bec::TableColumnsListBE::Comment
                                                                   row: rowIndex];
     [mColumnsComment setString: comments];
     
-    // Other column flag checkboxes outside the table view.
+    if (mBackEnd->is_editing_live_object())
     {
-      // Remove any old checkboxes.
-      int i, c = [mColumnFlagCheckboxes count];
-      for (i = 0; i < c; i++) {
-        [[mColumnFlagCheckboxes objectAtIndex: i] removeFromSuperview];
-      }
-      [mColumnFlagCheckboxes autorelease];
-      mColumnFlagCheckboxes = nil;
-    }
-    
-    NSArray* flags = [mColumnsDataSource columnFlagsForRow: rowIndex];
-    
-    {
-      // Add new checkboxes for the column flags.
-      mColumnFlagCheckboxes = [[NSMutableArray alloc] initWithCapacity: [flags count]];
-      NSRect r = [mColumnsCollation frame];
-      NSRect lastFlagRect;
-      r.size.height = 14;
-      r.origin.y -= 21;
-      int i, c = [flags count] / 2;
-      for (i = 0; i < c; i++) {
-        NSString* title = [flags objectAtIndex: i * 2];
-        NSCellStateValue state = ( [[flags objectAtIndex: i * 2 + 1] boolValue] ? NSOnState : NSOffState );
-        NSButton* b = [[[NSButton alloc] initWithFrame: r] autorelease];
-        [b setTag: i];
-        [b setButtonType: NSSwitchButton];
-        [b setTitle: title];
-        [b setState: state];
-        [b setFont: [NSFont systemFontOfSize: [NSFont smallSystemFontSize]]];
-        [[b cell] setControlSize: NSSmallControlSize];
-        [b sizeToFit];
-        [b setAutoresizingMask: NSViewMinYMargin];
-        [b setTarget: self];
-        [b setAction: @selector(userClickButton:)];
-        [mColumnsDetailsBox addSubview: b];
-        [mColumnFlagCheckboxes addObject: b];
-        r = NSOffsetRect(r, 0, -17);
-      }
-      lastFlagRect = r;
-      
-      [mColumnFlagsLabel setHidden: c == 0];
-      
-      {
-        // Resize the comments text area.
-        NSView* scrollView = [[mColumnsComment superview] superview];
-        NSRect commentsFrame = [scrollView frame];
-        float commentsHeight = lastFlagRect.origin.y + lastFlagRect.size.height - commentsFrame.origin.y;
-        commentsFrame.size.height = commentsHeight;
-        [scrollView setFrame: commentsFrame];
-        
-        NSRect labelFrame = [mColumnsCommentLabel frame];
-        labelFrame.origin.y = NSMaxY(commentsFrame) - labelFrame.size.height;
-        [mColumnsCommentLabel setFrame: labelFrame];
-      }
+      NSArray *flags = MArrayFromStringVector(mBackEnd->get_columns()->get_datatype_flags(rowIndex, true));
+      int flag;
+      mBackEnd->get_columns()->get_field(rowIndex, bec::TableColumnsListBE::IsPK, flag);
+      [mColumnsFlagPK setState: flag ? NSOnState : NSOffState];
+      mBackEnd->get_columns()->get_field(rowIndex, bec::TableColumnsListBE::IsNotNull, flag);
+      [mColumnsFlagNN setState: flag ? NSOnState : NSOffState];
+      mBackEnd->get_columns()->get_field(rowIndex, bec::TableColumnsListBE::IsUnique, flag);
+      [mColumnsFlagUNQ setState: flag ? NSOnState : NSOffState];
+
+      [mColumnsFlagBIN setEnabled: [flags containsObject: @"BINARY"]];
+      [mColumnsFlagBIN setState: mBackEnd->get_columns()->get_column_flag(rowIndex, "BINARY") ? NSOnState : NSOffState];
+      [mColumnsFlagUN setEnabled: [flags containsObject: @"UNSIGNED"]];
+      [mColumnsFlagUN setState: mBackEnd->get_columns()->get_column_flag(rowIndex, "UNSIGNED") ? NSOnState : NSOffState];
+      [mColumnsFlagZF setEnabled: [flags containsObject: @"ZEROFILL"]];
+      [mColumnsFlagZF setState: mBackEnd->get_columns()->get_column_flag(rowIndex, "ZEROFILL") ? NSOnState : NSOffState];
+
+      mBackEnd->get_columns()->get_field(rowIndex,MySQLTableColumnsListBE::IsAutoIncrementable, flag);
+      [mColumnsFlagAI setEnabled: flag];
+      mBackEnd->get_columns()->get_field(rowIndex, MySQLTableColumnsListBE::IsAutoIncrement, flag);
+      [mColumnsFlagAI setState: flag ? NSOnState : NSOffState];
     }
   }
 }
@@ -1112,6 +1091,13 @@ objectValueForItemAtIndex: (NSInteger) index;
 
 #pragma mark User interaction
 
+// Show FK editor or disable it by showing a placeholder with info depending on the currently
+// selected table engine.
+- (void) updateFKPlaceholder;
+{
+  [mFKWarningPanel setHidden: !mBackEnd->is_editing_live_object() || mBackEnd->engine_supports_foreign_keys()];
+}
+
 - (IBAction) userPickPopup: (id) sender;
 {
   NSString* popItemTitle = [sender titleOfSelectedItem];
@@ -1121,8 +1107,10 @@ objectValueForItemAtIndex: (NSInteger) index;
       popItemTitle = @" - ";
     mBackEnd->set_table_option_by_name("CHARACTER SET - COLLATE", [popItemTitle UTF8String]);
   }
-  else if (sender == mTableEngine) {
+  else if (sender == mTableEngine)
+  {
     mBackEnd->set_table_option_by_name("ENGINE", [popItemTitle UTF8String]);
+    [self updateFKPlaceholder];
   }
   else if( sender == mColumnsCollation) {
     int rowIndex = [mColumnsTable selectedRow];
@@ -1184,15 +1172,51 @@ objectValueForItemAtIndex: (NSInteger) index;
 }
 
 
+- (void)toggleHeader:(BOOL)flag
+{
+  float collapsedHeight = 38;
+  float expandedHeight = 130;
+  NSRect rect = [mEditorsView frame];
+  NSRect hrect = [mHeaderView frame];
+  NSRect trect = [mEditorsTabView frame];
+  
+  if (flag)
+    hrect.size.height = expandedHeight;
+  else
+    hrect.size.height = collapsedHeight;
+
+  hrect.origin.y = NSMaxY(rect) - NSHeight(hrect) + 1;
+  trect.size.height = NSHeight(rect) - NSMinY(trect) - NSHeight(hrect) + 1;
+  [mHeaderView setFrame: hrect];
+  [mEditorsTabView setFrame: trect];
+  [[mTableComment enclosingScrollView] setHidden: !flag];
+  
+  [[mHeaderView viewWithTag: 1] setFrame: flag ? NSMakeRect(9, NSHeight(hrect) - 48 - 9, 48, 48) : NSMakeRect(9, NSHeight(hrect) - 24 - 9, 24, 24)];
+    
+  for (id view in [mHeaderView subviews])
+  {
+    if ([view tag] >= 100)
+      [view setHidden: !flag];
+  }
+}
+
 
 - (IBAction) userClickButton: (id) sender;
 {
-  if ([mColumnFlagCheckboxes containsObject: sender]) {
-    // The button is one of the dynamically created checkboxes in the Columns section.
-    [mColumnsDataSource setFlag: ([sender state] == NSOnState ? YES : NO)
-                    forFlagName: [sender title]
-                  atColumnIndex: [mColumnsTable selectedRow]];
-  }
+  if (sender == mColumnsFlagPK)
+    mBackEnd->get_columns()->set_field([mColumnsTable selectedRow], bec::TableColumnsListBE::IsPK, [sender state] == NSOnState);
+  else if (sender == mColumnsFlagNN)
+    mBackEnd->get_columns()->set_field([mColumnsTable selectedRow], bec::TableColumnsListBE::IsNotNull, [sender state] == NSOnState);
+  else if (sender == mColumnsFlagUNQ)
+    mBackEnd->get_columns()->set_field([mColumnsTable selectedRow], bec::TableColumnsListBE::IsUnique, [sender state] == NSOnState);
+  else if (sender == mColumnsFlagBIN)
+    mBackEnd->get_columns()->set_field([mColumnsTable selectedRow], bec::TableColumnsListBE::IsBinary, [sender state] == NSOnState);
+  else if (sender == mColumnsFlagUN)
+    mBackEnd->get_columns()->set_field([mColumnsTable selectedRow], bec::TableColumnsListBE::IsUnsigned, [sender state] == NSOnState);
+  else if (sender == mColumnsFlagZF)
+    mBackEnd->get_columns()->set_field([mColumnsTable selectedRow], bec::TableColumnsListBE::IsZerofill, [sender state] == NSOnState);
+  else if (sender == mColumnsFlagAI)
+    mBackEnd->get_columns()->set_field([mColumnsTable selectedRow], MySQLTableColumnsListBE::IsAutoIncrement, [sender state] == NSOnState);
   else if (sender == mPartitionEnabledCheckbox) {
     mBackEnd->set_partition_type( ([mPartitionEnabledCheckbox state] == NSOnState ? "HASH" : "") );
     [self refreshTableEditorGUIPartitioningTab];
@@ -1220,6 +1244,10 @@ objectValueForItemAtIndex: (NSInteger) index;
                                      bec::FKConstraintListBE::ModelOnly,
                                      [sender state] == NSOnState);
   }
+  else if (sender == mHeaderExpander)
+  {
+    [self toggleHeader: [sender state] == NSOnState];
+  }
   else {
     NSAssert1(NO, @"DEBUG - User clicked unmatched button: '%@'", [sender title]);
   }
@@ -1243,8 +1271,6 @@ objectValueForItemAtIndex: (NSInteger) index;
   }
 }
 
-
-
 // Called by Cocoa after every keystroke in a text field.
 - (void) controlTextDidChange: (NSNotification*) aNotification;
 {
@@ -1296,6 +1322,15 @@ objectValueForItemAtIndex: (NSInteger) index;
       }
     }    
   }
+  else if (sender == mColumnsType)
+    mBackEnd->get_columns()->set_field([mColumnsTable selectedRow], bec::TableColumnsListBE::Type, [[sender stringValue] UTF8String]);
+  else if (sender == mColumnsName)
+    mBackEnd->get_columns()->set_field([mColumnsTable selectedRow], bec::TableColumnsListBE::Name, [[sender stringValue] UTF8String]);
+  else if (sender == mColumnsDefault)
+  {
+    mBackEnd->get_columns()->set_field([mColumnsTable selectedRow], bec::TableColumnsListBE::Default, [[sender stringValue] UTF8String]);
+    [mColumnsTable setNeedsDisplay: YES];
+  }
   else if (sender == mColumnsTable) {
     ; // Ignore.
   }
@@ -1411,6 +1446,14 @@ objectValueForItemAtIndex: (NSInteger) index;
   }
 }
 
+
+- (CGFloat)splitView:(NSSplitView *)splitView constrainMaxCoordinate:(CGFloat)proposedMax ofSubviewAt:(NSInteger)dividerIndex
+{
+  if (splitView == mColumnsSplitter && ![splitView isVertical])
+    return MIN(proposedMax, NSHeight([splitView frame]) - 150.0);
+  return proposedMax;
+}
+
 #pragma mark Super class overrides
 
 - (id) identifier;
@@ -1431,7 +1474,7 @@ objectValueForItemAtIndex: (NSInteger) index;
 
 - (NSView*) dockableView;
 {
-  return mEditorsTabView;
+  return mEditorsView;
 }
 
 
@@ -1442,6 +1485,12 @@ objectValueForItemAtIndex: (NSInteger) index;
 }
 
 
+- (void)pluginDidShow: (id)sender
+{
+  [[mEditorsView window] makeFirstResponder: mTableName];
+}
+
+
 - (BOOL)pluginWillClose: (id)sender
 {
   if (mEditorInsertsController && [mEditorInsertsController hasPendingChanges])
@@ -1491,28 +1540,52 @@ objectValueForItemAtIndex: (NSInteger) index;
 
 - (void)reinitWithArguments:(const grt::BaseListRef&)args
 {
+  BOOL isReinit = mBackEnd != 0;
+  
   [super reinitWithArguments: args];
   
   [[mEditorInsertsController view] removeFromSuperview];
   [mEditorInsertsController release];
   delete mBackEnd;
-  
+
   mBackEnd = new MySQLTableEditorBE(_grtm, db_mysql_TableRef::cast_from(args[0]), get_rdbms_for_db_object(args[0]));
+  
+  if (!isReinit)
+  {
+    if (!mBackEnd->is_editing_live_object())
+    {
+      mUnusedColumnsDetailsBox = mColumnsDetailsBox;
+      mColumnsDetailsBox = mColumnsDetailsBox2;
+      mColumnsCollation = mColumnsCollation2;
+      mColumnsComment = mColumnsComment2;
+      [mColumnsSplitter setVertical: YES];
+    }
+    else
+    {
+      mUnusedColumnsDetailsBox = mColumnsDetailsBox2;
+      [mColumnsSplitter setVertical: NO];
+    }
+    [mColumnsSplitter addSubview: mColumnsDetailsBox];
+  }
+  
+  [mSchemaName removeAllItems];
+  [mSchemaName addItemsWithTitles: MArrayFromStringVector(mBackEnd->get_all_schema_names())];
+  
   [mColumnsDataSource release];
-    mColumnsDataSource = [[MacTableEditorColumnsInformationSource alloc] initWithListModel: mBackEnd->get_columns()
-                                                                              tableBackEnd: mBackEnd];
+  mColumnsDataSource = [[MacTableEditorColumnsInformationSource alloc] initWithListModel: mBackEnd->get_columns()
+                                                                            tableBackEnd: mBackEnd];
   [mIndicesDataSource release];
-    mIndicesDataSource = [[MacTableEditorInformationSource alloc] initWithListModel: mBackEnd->get_indexes()];
+  mIndicesDataSource = [[MacTableEditorInformationSource alloc] initWithListModel: mBackEnd->get_indexes()];
   
   [mIndexColumnsDataSource release];
   mIndexColumnsDataSource = [[MacTableEditorIndexColumnsInformationSource alloc] initWithListModel: mBackEnd->get_indexes()->get_columns()
                               tableBackEnd: mBackEnd];
   
   [mFKDataSource release];
-    mFKDataSource = [[MacTableEditorInformationSource alloc] initWithListModel: mBackEnd->get_fks()];
+  mFKDataSource = [[MacTableEditorInformationSource alloc] initWithListModel: mBackEnd->get_fks()];
     
   [mFKColumnsDataSource release];
-    mFKColumnsDataSource = [[MacTableEditorFKColumnsInformationSource alloc] initWithListModel: mBackEnd->get_fks()->get_columns()
+  mFKColumnsDataSource = [[MacTableEditorFKColumnsInformationSource alloc] initWithListModel: mBackEnd->get_fks()->get_columns()
                                                                                   tableBackEnd: mBackEnd];
   
   if (!mBackEnd->is_editing_live_object())
@@ -1526,10 +1599,9 @@ objectValueForItemAtIndex: (NSInteger) index;
     }
     mEditorInsertsController = [[MResultsetViewer alloc] initWithRecordset: mBackEnd->get_inserts_model()];
     [mEditorInserts addSubview: [mEditorInsertsController view]];
-    NSRect rect = [mEditorInserts frame];
-    rect.origin.x = 0;
-    rect.origin.y = 0;
-    [[mEditorInsertsController view] setFrame: rect];
+    [mEditorInserts setAutoresizesSubviews: YES];
+    [[mEditorInsertsController view] setFrame: [mEditorInserts bounds]];
+    [[mEditorInsertsController view] setAutoresizingMask: NSViewMinXMargin|NSViewMaxXMargin|NSViewMinYMargin|NSViewMaxYMargin|NSViewWidthSizable|NSViewHeightSizable];
   }
   
   // Populate popup menus.
@@ -1580,11 +1652,13 @@ objectValueForItemAtIndex: (NSInteger) index;
     [item setView: [mPrivilegesTab view]];
     [item setLabel: @"Privileges"];
     [mEditorsTabView addTabViewItem: item];
+    [(MColoredView*)[mPrivilegesTab view] setBackgroundColor: [NSColor whiteColor]];
   }
   // Register a callback that will call [self refresh] when the edited object is
   // changed from somewhere else in the application.
   mBackEnd->set_refresh_ui_slot(boost::bind(call_refresh, self));    
   
+  [self updateFKPlaceholder];
   [self setupEditor];
   
   // Update the GUI.
@@ -1592,31 +1666,38 @@ objectValueForItemAtIndex: (NSInteger) index;
 }
 
 
-- (void) awakeFromNib;
+- (void)awakeFromNib;
 {
+  [mTabSwitcher setTabStyle: MEditorBottomTabSwitcher];
+  
+  // collapse header by default
+  [mHeaderExpander setState: NSOffState];
+  [self toggleHeader: NO];
+    
   // Store the min size specified in the .xib file.
-  [self setMinimumSize: [mEditorsTabView frame].size];
+  NSSize size = [mEditorsView frame].size;
+  [self setMinimumSize: size];
  
   // Assemle all the separate editor views into the tab view.
-  NSTabViewItem* item = [[[NSTabViewItem alloc] initWithIdentifier: @"table"] autorelease];
-  [item setView: mEditorTable];
-  [item setLabel: @"Table"];
-  [mEditorsTabView addTabViewItem: item];
+  NSTabViewItem* item;
   
   item = [[[NSTabViewItem alloc] initWithIdentifier: @"columns"] autorelease];
   [item setView: mEditorColumns];
   [item setLabel: @"Columns"];
   [mEditorsTabView addTabViewItem: item];
+  [mEditorColumns setBackgroundColor: [NSColor whiteColor]];
   
   item = [[[NSTabViewItem alloc] initWithIdentifier: @"indices"] autorelease];
   [item setView: mEditorIndices];
   [item setLabel: @"Indices"];
   [mEditorsTabView addTabViewItem: item];
+  [mEditorIndices setBackgroundColor: [NSColor whiteColor]];
   
   item = [[[NSTabViewItem alloc] initWithIdentifier: @"foreignkeys"] autorelease];
   [item setView: mEditorForeignKeys];
   [item setLabel: @"Foreign Keys"];
   [mEditorsTabView addTabViewItem: item];
+  [mEditorForeignKeys setBackgroundColor: [NSColor whiteColor]];
   
   item = [[[NSTabViewItem alloc] initWithIdentifier: @"triggers"] autorelease];
   [item setView: mEditorTriggers];
@@ -1627,9 +1708,10 @@ objectValueForItemAtIndex: (NSInteger) index;
   [item setView: mEditorPartitioning];
   [item setLabel: @"Partitioning"];
   [mEditorsTabView addTabViewItem: item];
+  [mEditorPartitioning setBackgroundColor: [NSColor whiteColor]];
   
   item = [[[NSTabViewItem alloc] initWithIdentifier: @"options"] autorelease];
-  NSScrollView* sv = [[NSScrollView alloc] initWithFrame: [mEditorTable frame]];
+  NSScrollView* sv = [[NSScrollView alloc] initWithFrame: [mEditorColumns frame]];
   [sv setDocumentView: mEditorOptions];
   [sv setHasHorizontalScroller: YES];
   [sv setHasVerticalScroller: YES];
@@ -1640,6 +1722,7 @@ objectValueForItemAtIndex: (NSInteger) index;
   [item setView: sv];
   [item setLabel: @"Options"];
   [mEditorsTabView addTabViewItem: item];
+  [mEditorOptions setBackgroundColor: [NSColor whiteColor]];
   
   [mColumnsTable registerForDraggedTypes: [NSArray arrayWithObject: @"com.sun.mysql.workbench.column"]];
   
@@ -1656,7 +1739,10 @@ objectValueForItemAtIndex: (NSInteger) index;
 //--------------------------------------------------------------------------------------------------
 
 - (void) dealloc;
-{
+{  
+  [[NSNotificationCenter defaultCenter] removeObserver: self];
+  [NSRunLoop cancelPreviousPerformRequestsWithTarget: self];
+  
   [mPrivilegesTab release];
   [mColumnsDataSource release];
   [mColumnTypes release];
@@ -1665,6 +1751,7 @@ objectValueForItemAtIndex: (NSInteger) index;
   [mFKDataSource release];
   [mPartitionsTreeDataSource release];
   [mFKColumnsDataSource release];
+  [mUnusedColumnsDetailsBox release];
   
   delete mBackEnd;
   
@@ -1795,6 +1882,12 @@ static int process_syntax_error(const int line, const int err_tok_line_pos, cons
   return mBackEnd;
 }
 
+
+- (NSImage*)titleIcon
+{
+  return [NSImage imageNamed: @"tab_icon_db.Table.png"];
+}
+
 @end
 
 
