@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2008, 2010, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2008, 2012, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -24,7 +24,9 @@ using System.Windows.Forms;
 
 using Aga.Controls.Tree;
 
+using MySQL.Forms;
 using MySQL.Grt;
+using MySQL.GUI.Workbench.Plugins.Properties;
 using MySQL.Utilities;
 
 namespace MySQL.GUI.Workbench.Plugins
@@ -35,7 +37,7 @@ namespace MySQL.GUI.Workbench.Plugins
   {
     #region Member Variables
 
-    private MySQLTableEditorBE tableEditorBE { get { return editorBE as MySQLTableEditorBE; } } 
+    private MySQLTableEditorBE tableEditorBE { get { return Backend as MySQLTableEditorBE; } } 
     private GrtListNode activeNode = null;
     private bool dragInProgress = false;
 
@@ -50,6 +52,8 @@ namespace MySQL.GUI.Workbench.Plugins
     private SimpleGrtTreeModel partitionTreeModel;
     private MySQLTablePartitionTreeBE partitionTreeBE;
 
+    private bool headingCollapsed = false;
+
     #endregion
 
     #region Constructors
@@ -70,6 +74,9 @@ namespace MySQL.GUI.Workbench.Plugins
       {
         InitFormData();
 
+        if (!IsEditingLiveObject)
+          AdjustToSmallerLayout();
+
         CallRefreshFormData();
       }
       finally
@@ -85,10 +92,20 @@ namespace MySQL.GUI.Workbench.Plugins
 
     #region ObjectEditorPlugin Overrides
 
+    protected override void CallRefreshFormData()
+    {
+        base.CallRefreshFormData();
+
+        // XXX: Kind of a hack since the base class doesn't care about anything other than mainTabControl.
+        topPanel.Parent = mainTabControl.Parent;
+    }
+
     public override bool ChangeGrtList(GrtManager GrtManager, GrtValue GrtList)
     {
       // Make sure the changes are stored
-      sqlEd.Parse(false);
+      // must be done synchronously otherwise the actual parsing will happen at a random time, after
+      // the editor obj is already deleted
+      sqlEd.Parse(true);
 
       InitializingControls = true;
       SuspendLayout();
@@ -98,9 +115,7 @@ namespace MySQL.GUI.Workbench.Plugins
         grtManager = GrtManager;
         grtList = GrtList;
 
-        if (editorBE != null)
-          editorBE.disable_auto_refresh();
-        editorBE = new MySQLTableEditorBE(GrtManager, GrtList);
+        Backend = new MySQLTableEditorBE(GrtManager, GrtList);
 
         InitFormData();
 
@@ -127,6 +142,8 @@ namespace MySQL.GUI.Workbench.Plugins
     {
       int current_page = mainTabControl.SelectedIndex;
 
+      schemaLabel.Text = tableEditorBE.get_schema_name();
+
       // Init Columns TreeView
       columnsTreeView.SelectedNode = null;
       if (columnsTreeView.Model != null && columnsTreeView.Model is DbMysqlTableColumnsListModel)
@@ -263,6 +280,32 @@ namespace MySQL.GUI.Workbench.Plugins
       partitionTreeView.Refresh();
     }
 
+    /// <summary>
+    /// The current layout is too big for the small room we have for the table editor
+    /// in the modeling section, so we switch to a smaller layout that uses less vertical space.
+    /// </summary>
+    void AdjustToSmallerLayout()
+    {
+      // Start with a collapsed header section.
+      collapsePictureBox_Click(null, null);
+
+      // Make the right part of the columns split container visible and dock collation and
+      // comment controls to it.
+      columnListSplitContainer.Panel2Collapsed = false;
+      label9.Parent = oldTableLayoutPanel;
+      oldTableLayoutPanel.SetCellPosition(label9, new TableLayoutPanelCellPosition(0, 0));
+      columnCollationComboBox.Parent = oldTableLayoutPanel;
+      oldTableLayoutPanel.SetCellPosition(columnCollationComboBox, new TableLayoutPanelCellPosition(1, 0));
+      label12.Parent = oldTableLayoutPanel;
+      label12.Anchor = AnchorStyles.Left | AnchorStyles.Top | AnchorStyles.Right;
+      oldTableLayoutPanel.SetCellPosition(label12, new TableLayoutPanelCellPosition(0, 1));
+      columnCommentTextBox.Parent = oldTableLayoutPanel;
+      oldTableLayoutPanel.SetCellPosition(columnCommentTextBox, new TableLayoutPanelCellPosition(1, 1));
+
+      // Finally hide the entire lower part (column details).
+      tableLayoutPanel1.Hide();
+    }
+
     void canEdit(object sender, Aga.Controls.Tree.NodeControls.NodeControlValueEventArgs e)
     {
       if (dragInProgress)
@@ -409,9 +452,6 @@ namespace MySQL.GUI.Workbench.Plugins
       columnsListModel.RefreshModel();
       indicesListModel.RefreshModel();
       fkListModel.RefreshModel();
-      // unnecessary and causes a crash when switching the edited table if called from here
-      //if (fkColumnsListModel != null)
-      //  fkColumnsListModel.RefreshModel();
 
       // partitioning
       if (tableEditorBE.get_partition_type() == null || tableEditorBE.get_partition_type() == "")
@@ -440,14 +480,6 @@ namespace MySQL.GUI.Workbench.Plugins
         insertsRecordsetView.Model = tableEditorBE.get_inserts_model();
         insertsRecordsetView.Model.refresh();
       }
-
-      if (InitializingControls)
-      {
-        // XXX this is kind of hackish. to streamline the workflow, we always select the 
-        // 1st tab of the table editor if its being shown for a new object
-        if (tableEditorBE.is_new_table())
-          mainTabControl.SelectedIndex = 0;
-      }
     }
 
     private void nameTextBox_TextChanged(object sender, EventArgs e)
@@ -471,26 +503,11 @@ namespace MySQL.GUI.Workbench.Plugins
         // Make sure a text change is now applied
         nameTextBox.CheckTextChangedNow();
 
-        if ((e.Shift && e.KeyCode == Keys.Tab) || e.KeyCode != Keys.Tab)
-          mainTabControl.SelectedTab = columnsTabPage;
-        else
-          ActiveControl = optComments;
-
         e.SuppressKeyPress = true;
         e.Handled = true;
       }
     }
 
-    private void commentsTextBox_KeyDown(object sender, KeyEventArgs e)
-    {
-      if ((e.KeyCode == Keys.Enter || e.KeyCode == Keys.Return) && e.Shift)
-      {
-        mainTabControl.SelectedTab = columnsTabPage;
-        ActiveControl = columnsTreeView;
-        nameNodeControl.BeginEdit();
-      }
-    }
-
     private void DbMysqlTableEditor_Load(object sender, EventArgs e)
     {
       ActiveControl = nameTextBox;
@@ -498,6 +515,8 @@ namespace MySQL.GUI.Workbench.Plugins
 
     private void mainTabControl_SelectedIndexChanged(object sender, EventArgs e)
     {
+      sectionLabel.Text = mainTabControl.SelectedTab.Text;
+
       if (mainTabControl.SelectedTab == columnsTabPage)
       {
         // if the columns tab is selected, auto-edit a table-column
@@ -524,8 +543,8 @@ namespace MySQL.GUI.Workbench.Plugins
       {
         fkListModel.RefreshModel();
 
-        if (!IsEditingLiveObject || optEngine.SelectedIndex == 0 || // Index 0 is the server default.
-          optEngine.Text.Equals("InnoDB", StringComparison.InvariantCultureIgnoreCase))
+        if (optEngine.SelectedIndex == 0 || // Index 0 is the server default.
+          tableEditorBE.engine_supports_foreign_keys())
         {
           foreignKeyWarningPanel.Visible = false;
 
@@ -560,13 +579,10 @@ namespace MySQL.GUI.Workbench.Plugins
           if (mainTabControl.SelectedIndex > 0)
             mainTabControl.SelectedIndex--;
         }
-        else if (e.KeyCode == Keys.Enter || e.KeyCode == Keys.Return)
-        {
-          Close();
-        }
       }
     }
 
+    #endregion
 
     #region Columns
 
@@ -602,55 +618,104 @@ namespace MySQL.GUI.Workbench.Plugins
 
     private void columnsTreeView_SelectionChanged(object sender, EventArgs e)
     {
-      if (activeNode != null)
-        activeNode = null; // make sure the following changes wont be commited
+      if (InsideInitializationOrRefresh())
+        return;
+
+      activeNode = null; // Make sure the following changes won't be committed.
 
       if (columnsTreeView.SelectedNode != null)
       {
-        activeNode = columnsTreeView.SelectedNode.Tag as GrtListNode;
+        NodeId nodeId = (columnsTreeView.SelectedNode.Tag as GrtListNode).NodeId;
 
-        if (activeNode != null)
+        string stringValue;
+        columnsListModel.GrtList.get_field(nodeId,
+          (int)MySQLTableColumnsListBE.MySQLColumnListColumns.Comment, out stringValue);
+        columnCommentTextBox.Text = stringValue;
+        columnCommentTextBox.Enabled = true;
+
+        // charset/collation
+        String hasCharset = null;
+        columnsListModel.GrtList.get_field(nodeId,
+          (int)MySQLTableColumnsListBE.MySQLColumnListColumns.HasCharset, out hasCharset);
+        if (hasCharset == "1")
         {
-          // Set column details
+          String column_cscoll;
+          columnsListModel.GrtList.get_field(nodeId,
+            (int)MySQLTableColumnsListBE.MySQLColumnListColumns.CharsetCollation, out column_cscoll);
 
-          // comment
-          string comment;
-          columnsListModel.GrtList.get_field(activeNode.NodeId,
-            (int)MySQLTableColumnsListBE.MySQLColumnListColumns.Comment, out comment);
-          columnCommentTextBox.Text = comment;
-          columnCommentTextBox.Enabled = true;
+          columnCollationComboBox.Enabled = true;
 
-          // charset/collation
-          String hasCharset = null;
-          columnsListModel.GrtList.get_field(activeNode.NodeId,
-            (int)MySQLTableColumnsListBE.MySQLColumnListColumns.HasCharset, out hasCharset);
-          if (hasCharset == "1")
+          int idx = 0;
+          columnCollationComboBox.SelectedIndex = 0;
+
+          foreach (String next_cscoll in columnCollationComboBox.Items)
           {
-            String column_cscoll;
-            columnsListModel.GrtList.get_field(activeNode.NodeId,
-              (int)MySQLTableColumnsListBE.MySQLColumnListColumns.CharsetCollation, out column_cscoll);
-
-            columnCollationComboBox.Enabled = true;
-
-            int idx = 0;
-            columnCollationComboBox.SelectedIndex = 0;
-
-            foreach (String next_cscoll in columnCollationComboBox.Items)
+            if (next_cscoll == column_cscoll)
             {
-              if (next_cscoll == column_cscoll)
-              {
-                columnCollationComboBox.SelectedIndex = idx;
-                break;
-              }
-              idx++;
+              columnCollationComboBox.SelectedIndex = idx;
+              break;
             }
-          }
-          else
-          {
-            columnCollationComboBox.SelectedIndex = 0;
-            columnCollationComboBox.Enabled = false;
+            idx++;
           }
         }
+        else
+        {
+          columnCollationComboBox.SelectedIndex = 0;
+          columnCollationComboBox.Enabled = false;
+        }
+
+        columnsListModel.GrtList.get_field(nodeId,
+          (int)MySQLTableColumnsListBE.MySQLColumnListColumns.Name, out stringValue);
+        columnNameTextBox.Text = stringValue;
+        columnNameTextBox.Enabled = true;
+
+        columnsListModel.GrtList.get_field(nodeId,
+          (int)MySQLTableColumnsListBE.MySQLColumnListColumns.Type, out stringValue);
+        columnDataTypeTextBox.Text = stringValue;
+        columnDataTypeTextBox.Enabled = true;
+
+        columnsListModel.GrtList.get_field(nodeId,
+          (int)MySQLTableColumnsListBE.MySQLColumnListColumns.Default, out stringValue);
+        columnDefaultTextBox.Text = stringValue;
+        columnDefaultTextBox.Enabled = true;
+
+        int flag;
+        columnsListModel.GrtList.get_field(nodeId,
+          (int)MySQLTableColumnsListBE.MySQLColumnListColumns.IsPK, out flag);
+        pkCheckBox.Checked = flag != 0;
+        pkCheckBox.Enabled = true;
+
+        columnsListModel.GrtList.get_field(nodeId,
+          (int)MySQLTableColumnsListBE.MySQLColumnListColumns.IsNotNull, out flag);
+        nnCheckBox.Checked = flag != 0;
+        nnCheckBox.Enabled = true;
+
+        columnsListModel.GrtList.get_field(nodeId,
+          (int)MySQLTableColumnsListBE.MySQLColumnListColumns.IsUnique, out flag);
+        uniqueCheckBox.Checked = flag != 0;
+        uniqueCheckBox.Enabled = true;
+
+        columnsListModel.GrtList.get_field(nodeId,
+          (int)MySQLTableColumnsListBE.MySQLColumnListColumns.IsBinary, out flag);
+        binaryCheckBox.Checked = flag != 0;
+        binaryCheckBox.Enabled = true;
+
+        columnsListModel.GrtList.get_field(nodeId,
+          (int)MySQLTableColumnsListBE.MySQLColumnListColumns.IsUnsigned, out flag);
+        unsignedCheckBox.Checked = flag != 0;
+        unsignedCheckBox.Enabled = true;
+
+        columnsListModel.GrtList.get_field(nodeId,
+          (int)MySQLTableColumnsListBE.MySQLColumnListColumns.IsZerofill, out flag);
+        zeroFillCheckBox.Checked = flag != 0;
+        zeroFillCheckBox.Enabled = true;
+
+        columnsListModel.GrtList.get_field(nodeId,
+          (int)MySQLTableColumnsListBE.MySQLColumnListColumns.IsAutoIncrement, out flag);
+        aiCheckBox.Checked = flag != 0;
+        aiCheckBox.Enabled = true;
+
+        activeNode = columnsTreeView.SelectedNode.Tag as GrtListNode;
       }
       else
       {
@@ -658,6 +723,26 @@ namespace MySQL.GUI.Workbench.Plugins
         columnCommentTextBox.Enabled = false;
         columnCommentTextBox.Text = "";
         columnCollationComboBox.Enabled = false;
+        columnNameTextBox.Text = "";
+        columnNameTextBox.Enabled = false;
+        columnDataTypeTextBox.Text = "";
+        columnDataTypeTextBox.Enabled = false;
+        columnDefaultTextBox.Text = "";
+        columnDefaultTextBox.Enabled = false;
+        pkCheckBox.Checked = false;
+        pkCheckBox.Enabled = false;
+        nnCheckBox.Checked = false;
+        nnCheckBox.Enabled = false;
+        uniqueCheckBox.Checked = false;
+        uniqueCheckBox.Enabled = false;
+        binaryCheckBox.Checked = false;
+        binaryCheckBox.Enabled = false;
+        unsignedCheckBox.Checked = false;
+        unsignedCheckBox.Enabled = false;
+        zeroFillCheckBox.Checked = false;
+        zeroFillCheckBox.Enabled = false;
+        aiCheckBox.Checked = false;
+        aiCheckBox.Enabled = false;
       }
     }
 
@@ -680,17 +765,125 @@ namespace MySQL.GUI.Workbench.Plugins
       }
     }
 
-    private void columnCommentTextBox_TextChanged(object sender, EventArgs e)
+    private void updateColumnTextField(MySQLTableColumnsListBE.MySQLColumnListColumns column, TextBox box)
     {
-      // column comment
       if (activeNode != null)
-        columnsListModel.GrtList.set_field(activeNode.NodeId,
-          (int)MySQLTableColumnsListBE.MySQLColumnListColumns.Comment, columnCommentTextBox.Text);
+      {
+        // Change an attribute triggers a whole chain of events which ultimately cause the
+        // table editor to refresh completely (including all nodes in the columns tree etc.).
+        // So we have to take measure to restore what was active when the user changed the value.
+        NodeId nodeId = activeNode.NodeId;
+        if (!columnsListModel.GrtList.set_field(nodeId, (int)column, box.Text))
+        {
+          CustomMessageBox.Show(MessageType.MessageError, "Could not set new value",
+            "The entered value contains errors and cannot be accepted. The previous value is kept instead.",
+            "Close");
+
+          string originalText;
+          columnsListModel.GrtList.get_field(nodeId, (int)column, out originalText);
+          box.Text = originalText;
+        }
+
+        columnsTreeView.SelectedNode = columnsTreeView.Root.Children[nodeId.get_by_index(0)];
+      }
+    }
+
+    private void applyColumnText(TextBox box)
+    {
+      NodeId nodeId = activeNode.NodeId;
+      MySQLTableColumnsListBE.MySQLColumnListColumns column = MySQLTableColumnsListBE.MySQLColumnListColumns.IsPK;
+      switch (box.Tag.ToString())
+      {
+        case "0":
+          column = MySQLTableColumnsListBE.MySQLColumnListColumns.Name;
+          break;
+        case "1":
+          column = MySQLTableColumnsListBE.MySQLColumnListColumns.Comment;
+          break;
+        case "2":
+          column = MySQLTableColumnsListBE.MySQLColumnListColumns.Type;
+          break;
+        case "3":
+          column = MySQLTableColumnsListBE.MySQLColumnListColumns.Default;
+          break;
+      }
+      if (column != MySQLTableColumnsListBE.MySQLColumnListColumns.IsPK)
+      {
+        updateColumnTextField(column, box);
+        //box.Focus();
+        box.SelectAll();
+      }
+    }
+
+    private void columnTextBox_Leave(object sender, EventArgs e)
+    {
+      if (activeNode != null && !IsDisposed)
+        applyColumnText(sender as TextBox);
+    }
+
+    private void columnTextBox_KeyDown(object sender, KeyEventArgs e)
+    {
+      if (activeNode != null && (e.KeyCode == Keys.Enter || e.KeyCode == Keys.Return))
+      {
+        applyColumnText(sender as TextBox);
+
+        e.SuppressKeyPress = true;
+        e.Handled = true;
+      }
+    }
+
+    private void columnFlagsChanged(object sender, EventArgs e)
+    {
+      if (activeNode == null)
+        return;
+
+      NodeId nodeId = activeNode.NodeId;
+      CheckBox box = sender as CheckBox;
+      int value = box.Checked ? 1 : 0;
+      MySQLTableColumnsListBE.MySQLColumnListColumns columnValue = MySQLTableColumnsListBE.MySQLColumnListColumns.Name;
+      switch (box.Tag.ToString())
+      {
+        case "0":
+          columnValue =  MySQLTableColumnsListBE.MySQLColumnListColumns.IsPK;
+          break;
+        case "1":
+          columnValue = MySQLTableColumnsListBE.MySQLColumnListColumns.IsNotNull;
+          break;
+        case "2":
+          columnValue = MySQLTableColumnsListBE.MySQLColumnListColumns.IsUnique;
+          break;
+        case "3":
+          columnValue = MySQLTableColumnsListBE.MySQLColumnListColumns.IsBinary;
+          break;
+        case "4":
+          columnValue = MySQLTableColumnsListBE.MySQLColumnListColumns.IsUnsigned;
+          break;
+        case "5":
+          columnValue = MySQLTableColumnsListBE.MySQLColumnListColumns.IsZerofill;
+          break;
+        case "6":
+          columnValue = MySQLTableColumnsListBE.MySQLColumnListColumns.IsAutoIncrement;
+          break;
+      }
+
+      if (columnValue != MySQLTableColumnsListBE.MySQLColumnListColumns.Name)
+      {
+        if (!columnsListModel.GrtList.set_field(activeNode.NodeId, (int)columnValue, value))
+        {
+          // Value not accepted. Restore previous one.
+          columnsListModel.GrtList.get_field(activeNode.NodeId, (int)columnValue, out value);
+          box.Checked = value != 0;
+        }
+      }
+
+      columnsTreeView.SelectedNode = columnsTreeView.Root.Children[nodeId.get_by_index(0)];
+      box.Focus();
     }
 
     #endregion
 
     #region Index
+
     private void indicesTreeView_SelectionChanged(object sender, EventArgs e)
     {
       // remove old virtual value events
@@ -1005,7 +1198,8 @@ namespace MySQL.GUI.Workbench.Plugins
     private void nameTextBox_Leave(object sender, EventArgs e)
     {
       // Make sure a text change is now applied
-      nameTextBox.CheckTextChangedNow();
+      if (!IsDisposed)
+        nameTextBox.CheckTextChangedNow();
     }
 
     private void DbMysqlTableEditor_KeyDown(object sender, KeyEventArgs e)
@@ -1031,6 +1225,11 @@ namespace MySQL.GUI.Workbench.Plugins
       }
     }
 
+    private void DbMysqlTableEditor_Shown(object sender, EventArgs e)
+    {
+      nameTextBox.Focus();
+    }
+
     private bool forceClosing = false;
     public override void Close(bool force)
     {
@@ -1392,7 +1591,6 @@ namespace MySQL.GUI.Workbench.Plugins
       if (null == insertsRecordsetView)
       {
         insertsRecordsetView = new MySQL.Grt.Db.RecordsetView();
-        insertsRecordsetView.PinButtonVisible = false;
         insertsRecordsetView.Embed(insertsTabPage, tableEditorBE.get_inserts_model());
       }
       insertsRecordsetView.Model.refresh();
@@ -1413,6 +1611,7 @@ namespace MySQL.GUI.Workbench.Plugins
 
     #endregion
 
+    #region Event handling
 
     void default_NodeMouseDoubleClick(object sender, Aga.Controls.Tree.TreeNodeAdvMouseEventArgs e)
     {
@@ -1431,6 +1630,70 @@ namespace MySQL.GUI.Workbench.Plugins
       }
     }
 
+    private void fkColumnsTreeView_Click(object sender, EventArgs e)
+    {
+      targetColumnFkNodeControl.BeginEdit();
+    }
+
+    private void collapsePictureBox_Click(object sender, EventArgs e)
+    {
+      headingCollapsed = !headingCollapsed;
+      if (headingCollapsed)
+      {
+        collapsePictureBox.Image = Resources.EditorExpand;
+        headingLayoutPanel.AutoSize = false;
+        headingLayoutPanel.Size = new System.Drawing.Size(headingLayoutPanel.Width, 38);
+        headingLayoutPanel.SetRowSpan(this.pictureBox1, 1);
+      }
+      else
+      {
+        collapsePictureBox.Image = Resources.EditorCollapse;
+        headingLayoutPanel.AutoSize = true;
+        headingLayoutPanel.SetRowSpan(this.pictureBox1, 2);
+      }
+    }
+
+    private void undoItem_Click(object sender, EventArgs e)
+    {
+      sqlEd.sqlEditor.UndoRedo.Undo();
+    }
+
+    private void cutItem_Click(object sender, EventArgs e)
+    {
+      sqlEd.sqlEditor.Clipboard.Cut();
+    }
+
+    private void copyItem_Click(object sender, EventArgs e)
+    {
+      sqlEd.sqlEditor.Clipboard.Copy();
+    }
+
+    private void pasteItem_Click(object sender, EventArgs e)
+    {
+      sqlEd.sqlEditor.Clipboard.Paste();
+    }
+
+    private void selectAllItem_Click(object sender, EventArgs e)
+    {
+      sqlEd.sqlEditor.Selection.SelectAll();
+    }
+
+    private void redoItem_Click(object sender, EventArgs e)
+    {
+      sqlEd.sqlEditor.UndoRedo.Redo();
+    }
+
+    private void editorContextMenu_Opening(object sender, CancelEventArgs e)
+    {
+      // Enable/Disable menu items depending on the edit content.
+      undoItem.Enabled = sqlEd.sqlEditor.UndoRedo.CanUndo;
+      redoItem.Enabled = sqlEd.sqlEditor.UndoRedo.CanRedo;
+      cutItem.Enabled = sqlEd.sqlEditor.Selection.Length > 0;
+      copyItem.Enabled = sqlEd.sqlEditor.Selection.Length > 0;
+      pasteItem.Enabled = sqlEd.sqlEditor.Clipboard.CanPaste;
+      selectAllItem.Enabled = sqlEd.sqlEditor.TextLength > 0;
+    }
+
     #endregion
 
     #region Drag & Drop
@@ -1503,6 +1766,10 @@ namespace MySQL.GUI.Workbench.Plugins
         return;
       }
 
+      TreeNodeAdv dropNode = columnsTreeView.DropPosition.Node as TreeNodeAdv;
+      if (dropNode == null)
+        return;
+
       // Each time a node is moved in the backend the model recreates all nodes in the tree.
       // So this list of nodes we get from the data object is invalid right after the first move.
       // To make it still work we keep only the indices to change and update that list as we go.
@@ -1510,7 +1777,6 @@ namespace MySQL.GUI.Workbench.Plugins
       List<int> indices = new List<int>(nodes.Length);
       foreach (TreeNodeAdv nextNode in nodes)
         indices.Add(nextNode.Index);
-      TreeNodeAdv dropNode = columnsTreeView.DropPosition.Node as TreeNodeAdv;
 
       int targetIndex = dropNode.Index;
       if (columnsTreeView.DropPosition.Position == NodePosition.After)
@@ -1523,52 +1789,16 @@ namespace MySQL.GUI.Workbench.Plugins
     {
       columnsTreeView.HideEditor(); // In case a node editor just popped up.
       dragInProgress = true; // In case a new edit operation is just about to start.
-      columnsTreeView.DoDragDropSelectedNodes(DragDropEffects.Move);
+
+      // Remove the last node (row) from the selection as this is a virtual entry not meant
+      // to be moved or anything other than holding place for a new column entry.
+      columnsTreeView.Root.Children[columnsTreeView.Root.Children.Count - 1].IsSelected = false;
+      if (columnsTreeView.SelectedNodes.Count > 0)
+        columnsTreeView.DoDragDropSelectedNodes(DragDropEffects.Move);
       dragInProgress = false;
     }
 
     #endregion
 
-    private void undoItem_Click(object sender, EventArgs e)
-    {
-      sqlEd.sqlEditor.UndoRedo.Undo();
-    }
-
-    private void cutItem_Click(object sender, EventArgs e)
-    {
-      sqlEd.sqlEditor.Clipboard.Cut();
-    }
-
-    private void copyItem_Click(object sender, EventArgs e)
-    {
-      sqlEd.sqlEditor.Clipboard.Copy();
-    }
-
-    private void pasteItem_Click(object sender, EventArgs e)
-    {
-      sqlEd.sqlEditor.Clipboard.Paste();
-    }
-
-    private void selectAllItem_Click(object sender, EventArgs e)
-    {
-      sqlEd.sqlEditor.Selection.SelectAll();
-    }
-
-    private void redoItem_Click(object sender, EventArgs e)
-    {
-      sqlEd.sqlEditor.UndoRedo.Redo();
-    }
-
-    private void editorContextMenu_Opening(object sender, CancelEventArgs e)
-    {
-      // Enable/Disable menu items depending on the edit content.
-      undoItem.Enabled = sqlEd.sqlEditor.UndoRedo.CanUndo;
-      redoItem.Enabled = sqlEd.sqlEditor.UndoRedo.CanRedo;
-      cutItem.Enabled = sqlEd.sqlEditor.Selection.Length > 0;
-      copyItem.Enabled = sqlEd.sqlEditor.Selection.Length > 0;
-      pasteItem.Enabled = sqlEd.sqlEditor.Clipboard.CanPaste;
-      selectAllItem.Enabled = sqlEd.sqlEditor.TextLength > 0;
-    }
-
   }
-}
\ No newline at end of file
+}
